/*
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.tbgf.games.desdemona.client;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.BorderFactory;
import javax.swing.JPanel;

/**
 *
 * @author Mike
 */
public class BoardSquare extends JPanel implements MouseListener {

    protected Color color;
    private final DesdemonaUI desdemonaUI;
    public final byte x;
    public final byte y;
    public BoardSquare(DesdemonaUI ui,byte x, byte y) {
        this.desdemonaUI = ui;
        color = Color.gray;
        setBackground(Color.gray);
        this.setPreferredSize(new Dimension(100, 100));
        setBorder(BorderFactory.createLineBorder(Color.black));
        addMouseListener(this);
        this.x = x;
        this.y = y;
    }

    public synchronized void setColor(Color color) {
        this.color = color;
    }

    @Override
    public synchronized void paintComponent(Graphics g) {

        //Set AntiAliasing on, the pieces look SOO much better with it on.
        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        super.paintComponent(g);

        int height = getHeight();
        int width = getWidth();
        int diameter;

        if (height <= width) {
            diameter = height / 2;
        } else {
            diameter = width / 2;
        }
        g.setColor(color);
        Point topLeft = new Point();
        topLeft.x = (width - diameter) / 2;
        topLeft.y = (height - diameter) / 2;
        g.fillOval(topLeft.x, topLeft.y, diameter, diameter);
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
        desdemonaUI.boardSquareClicked(this);
    }

    public void mouseEntered(MouseEvent e) {
    }
}
