/*
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.tbgf.games.desdemona.client;

import net.sf.goodgame.tbgf.client.*;
import javax.swing.JOptionPane;
import net.sf.goodgame.shared.games.GameOptions;
import net.sf.goodgame.shared.games.Player;
import net.sf.goodgame.tbgf.games.desdemona.shared.Piece;

/**
 *
 * @author mchaten
 */
public class DesdemonaClient extends TurnBasedGameClient {

    private String name = "";
    @Override
    public void destroy() {
    }

    @Override
    protected String getGameTitle() {
        return "Desdemona: "+name;
    }

    @Override
    protected void initializeGame() {
        game = new DesdemonaClientEngine();
        game.setGameClient(this);
    }

    @Override
    protected void initializeUI() {
        game.initializeUI();
        this.rootPanel = game.getGamePanel();
    }

    @Override
    public GameOptions setupNewGame(GameOptions options) {
        name = JOptionPane.showInputDialog("What name should the game have?");
        options.put("name", name);
        return options;
    }

    @Override
    protected void gameIsSetup(GameOptions options) {
        byte size = options.getAsByte("size");
        name = options.getAsString("name");
        ((DesdemonaClientEngine) game).setSize(size);
        if (options.containsKey("spectator")) {
            Player currentTurn = new Player(options.getAsString("current"));
            Player blackPlayer = new Player(options.getAsString("black"));
            Player whitePlayer = new Player(options.getAsString("white"));
            byte[] board = options.getAsArray("spectator");
            Piece[][] gameBoard = new Piece[size][size];
            for (byte i = 0; i < size; i++) {
                for (byte j = 0; j < size; j++) {
                    byte piece = board[i * size + j];
                    if (piece == -1) {
                        gameBoard[i][j] = null;
                    } else {
                        gameBoard[i][j] = Piece.values()[piece];
                    }
                }
            }
            ((DesdemonaClientEngine) game).spectate(gameBoard, whitePlayer, blackPlayer, currentTurn, gameListener.whoAmI());
        }
    }

    @Override
    public void gameOver(String message) {
        ((DesdemonaClientEngine) game).gameOver(message);
    }

    @Override
    public void gameOver(byte[] message) {
        //not called
    }

    @Override
    protected void setGameOptions(GameOptions options) {
    }
}
