/*
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.tbgf.games.desdemona.client;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import javax.swing.JOptionPane;
import net.sf.goodgame.shared.InvalidConfigurationException;
import net.sf.goodgame.tbgf.client.DefaultTurnBasedGameUI;
import net.sf.goodgame.tbgf.games.desdemona.shared.GameBoard;
import net.sf.goodgame.tbgf.games.desdemona.shared.Move;
import net.sf.goodgame.tbgf.games.desdemona.shared.Piece;

/**
 *
 * @author Mike
 */
public class DesdemonaUI extends DefaultTurnBasedGameUI {

    private BoardSquare pieces[][];
    private final DesdemonaClientEngine engine;
    private boolean gameFinished;
    private GameBoard board;

    public DesdemonaUI(final DesdemonaClientEngine engine, final DesdemonaClient gameClient) {
        super(gameClient);
        this.engine = engine;
        gameFinished = false;
    }

    public void boardSquareClicked(BoardSquare square) {
        if (!gameFinished) {
            engine.tryMove(square.x, square.y);
        }
    }

    public synchronized void setPiece(byte x, byte y, Piece p) {
        board.setPiece(x, y, p);
        pieces[y][x].setColor(p == Piece.black ? Color.black : Color.white);
        repaint();
    }

    public synchronized Piece getPiece(byte x, byte y) {
        return board.getPiece(x, y);
    }

    public byte getBoardSize() {
        return board.getSize();
    }

    public void gameOver(String winner) {
        gameFinished = true;
        JOptionPane.showMessageDialog(this, winner + " Game Over!");
    }

    public void setBoardSize(byte size) throws InvalidConfigurationException {
        System.out.println("Set Board Size: " + size);
        board = new GameBoard(size);
        GridLayout layout = new GridLayout(size, size);
        layout.setHgap(2);
        layout.setVgap(2);
        gameBoard.setLayout(layout);
        pieces = new BoardSquare[size][size];
        for (byte i = 0; i < size; i++) {
            for (byte j = 0; j < size; j++) {
                pieces[j][i] = new BoardSquare(this, i, j);
                Piece cur = board.getPiece(i, j);
                if (cur != null) {
                    pieces[j][i].setColor(cur == Piece.black ? Color.black : Color.white);
                }
                gameBoard.add(pieces[j][i]);
            }
        }
        gameBoard.setPreferredSize(new Dimension(50 * size, 50 * size));
        initUI();
    }

    public void showValidMoves(Move[] moves) {
        for (Move m : moves) {
            BoardSquare square = pieces[m.Y][m.X];
            square.setBackground(Color.green);
        }
    }

    public void clearValidMoves() {
        for (BoardSquare[] squares : pieces) {
            for (BoardSquare square : squares) {
                square.setBackground(Color.gray);
            }
        }
    }
}
