/*
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.tbgf.games.desdemona.shared;

import net.sf.goodgame.shared.InvalidConfigurationException;

/**
 *
 * @author mchaten
 */
public class GameBoard {

    protected Piece[][] board;
    protected byte size;
    public GameBoard(byte size) throws InvalidConfigurationException
    {
        if(size % 2 == 0 && size <= 12 && size >= 6)
            board = new Piece[size][size];
        else
            throw new InvalidConfigurationException("Size: "+size+" is not appropriate");
        this.size = size;
        setPiece((byte)(size/2),(byte)(size/2),Piece.black);
        setPiece((byte)(size/2-1),(byte)(size/2-1),Piece.black);
        setPiece((byte)(size/2-1),(byte)(size/2),Piece.white);
        setPiece((byte)(size/2),(byte)(size/2-1),Piece.white);
    }
   
    public byte getSize()
    {
        return size;
    }
    public Piece getPiece(byte x, byte y)
    {
        if(x >= size || y >= size)
            return null;
        return board[x][y];
    }
    public void setPiece(byte x, byte y, Piece p)
    {
        if(x >= size || y >= size)
        {
            System.out.println("FIXME: Attempting to set piece at an invalid index");
            return;
        }
            
        board[x][y] = p;
    }
    public Piece hasMostPieces()
    {
        short blackPieces = 0;
        short whitePieces = 0;
        for(byte x = 0;x<size;x++)
        {
            for(byte y = 0;y<size;y++)
            {
                if(board[x][y] == Piece.black)
                    blackPieces++;
                else if(board[x][y] == Piece.white)
                    whitePieces++;
            }
        }
        if(blackPieces == whitePieces)
            return null;
        return blackPieces > whitePieces?Piece.black:Piece.white;
    }
}
