/*
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.tbgf.client;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import net.sf.goodgame.client.gui.chat.ChatBox;
import net.sf.goodgame.client.gui.translucency.TranslucentJButton;
import net.sf.goodgame.client.gui.translucency.TranslucentJPanel;
import net.sf.goodgame.client.user.friends.FriendsList;
import net.sf.goodgame.client.user.friends.FriendsListGUI;
import net.sf.goodgame.client.user.UserList;
import net.sf.goodgame.shared.UserType;

/**
 *
 * @author mchaten
 */
public abstract class DefaultTurnBasedGameUI extends TranslucentJPanel {

    protected TranslucentJPanel gameBoard;
    private TranslucentJPanel gameBoardAndChatBoxPanel;
    private JPanel userListPanel;
    private final TurnBasedGameClient client;

    public DefaultTurnBasedGameUI(final TurnBasedGameClient client) {
        super(0.0f);
        setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
        setPreferredSize(new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE));
        gameBoardAndChatBoxPanel = new TranslucentJPanel(0.5f);
        gameBoard = new TranslucentJPanel(0.0f);
        userListPanel = new TranslucentJPanel(0.5f);
        this.client = client;
    }

    protected void initUI() {
        ChatBox cb = client.getChatBox();
        UserList ul = client.getUserList();

        userListPanel.setLayout(new BoxLayout(userListPanel, BoxLayout.PAGE_AXIS));
        userListPanel.setPreferredSize(new Dimension(200, Integer.MAX_VALUE));
        userListPanel.setMaximumSize(new Dimension(300, Integer.MAX_VALUE));
        userListPanel.add(ul.getComponent());
        JPanel buttonPanel = new TranslucentJPanel(0.0f);
        JButton leave = new TranslucentJButton("Leave Game", 0.8f);

        leave.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                client.leaveGame();
            }
        });
        if (client.gameListener.getMyType() != UserType.guest) {
            final FriendsListGUI friendsList = new FriendsListGUI(client.gameListener.getPrivateChatController(), client.io.getOutgoingConnection());
            friendsList.setFriendsList(client.gameListener.getFriendsList());
            JButton friends = new TranslucentJButton("Friends", 0.8f);
            friends.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    friendsList.show();
                }
            });
            buttonPanel.add(friends);
        }
        buttonPanel.add(leave);
        userListPanel.add(buttonPanel);
        gameBoardAndChatBoxPanel.setLayout(new BoxLayout(gameBoardAndChatBoxPanel, BoxLayout.PAGE_AXIS));
        gameBoardAndChatBoxPanel.add(gameBoard);
        gameBoardAndChatBoxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        gameBoardAndChatBoxPanel.add(cb);
        add(gameBoardAndChatBoxPanel);
        add(Box.createRigidArea(new Dimension(10, 0)));
        add(userListPanel);
        validate();
    }
}
