/*
(C) 2008 Michael Chaten
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.game;

import net.sf.goodgame.client.gametype.GameType;
import java.util.HashMap;
import java.util.Map;
import net.sf.goodgame.shared.games.GameState;
import net.sf.goodgame.shared.games.table.GameTableRow;

public class Game {

    private static final Map<Short, Game> gameMap = new HashMap<Short, Game>();
    public final short gameID;
    private volatile GameState state;
    private final GameType gametype;
    private GameTableRow row;

    public Game(short gameID, GameType gt, GameState state,  GameTableRow row) {
        this.gameID = gameID;
        this.state = state;
        gametype = gt;
        this.row = row;

        //This should be last always.
        gameMap.put(gameID, this);
    }

    public void setState(GameState state) {
        this.state = state;
    }

    public GameState getState() {
        return state;
    }

    public GameType getType() {
        return gametype;
    }

    public static Game getGame(short id) {
        return gameMap.get(id);
    }

    public static void clearGames() {
        gameMap.clear();
    }

    public GameTableRow getRow() {
        return row;
    }

    public void setRow(GameTableRow row) {
        this.row = row;
    }
}
