/*
(C) 2008 Michael Chaten, yura.net
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.game;

import javax.swing.JComponent;
import javax.swing.ListSelectionModel;
import net.sf.goodgame.client.gui.table.button.ButtonEditor;
import net.sf.goodgame.client.gui.table.button.ButtonRenderer;
import net.sf.goodgame.client.gui.translucency.TranslucentJTable;
import net.sf.goodgame.client.networking.OutgoingConnection;
import net.sf.goodgame.shared.games.table.GameTableColumn;
import net.sf.goodgame.shared.games.table.GameTableLayout;

public class GamesTable extends TranslucentJTable implements GameList {

    private GamesTableModel tableModel;
    private ButtonEditor editor;

    public GamesTable(GamesTableModel model, OutgoingConnection connection) {
        super(model, 0.5f);
        tableModel = model;
        this.getTableHeader().setReorderingAllowed(false);
        this.getTableHeader().setOpaque(false);
        this.getTableHeader().setDefaultRenderer(new GameTableCellRenderer());
        this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.setRowHeight(20);

        editor = new ButtonEditor(connection);
    //this.setDefaultRenderer(Game.class, new ButtonRenderer(editor));
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        return true;
    }

    public void addGame(Game game) {
        tableModel.addGame(game);
    }

    public void removeGame(Game game) {
        tableModel.removeGame(game);
    }

    public void updateGame(Game game) {
        tableModel.updateGame(game);
    }

    public void reset() {
        tableModel.clear();
    }

    public JComponent getComponent() {
        return this;
    }

    public void setGameTableLayout(GameTableLayout layout) {
        tableModel.setLayout(layout);
        for(int i = 0;i<tableModel.getColumnCount()-1;i++)
        {
            getColumnModel().getColumn(i).setCellRenderer(this.getDefaultRenderer(String.class));
        }
        getColumnModel().getColumn(tableModel.getColumnCount()-1).setCellRenderer(new ButtonRenderer(editor));
        getColumnModel().getColumn(tableModel.getColumnCount()-1).setCellEditor(editor);
    }
}
