/*
(C) 2008 Michael Chaten, yura.net
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.game;

import java.util.Vector;
import javax.swing.table.AbstractTableModel;
import net.sf.goodgame.shared.games.table.GameTableLayout;

/**
 *
 * @author mchaten
 */
public class GamesTableModel extends AbstractTableModel {

    private Vector<Game> games;
    private GameTableLayout layout;
    public GamesTableModel() {
        games = new Vector<Game>();
        layout = null;
    }

    public GameTableLayout getLayout() {
        return layout;
    }

    public void setLayout(GameTableLayout layout) {
        this.layout = layout;
        this.fireTableStructureChanged();
    }

    @Override
    public int getColumnCount() {
        return layout==null?1:layout.getColumns().length +1;
    }

    @Override
    public String getColumnName(int column) {
        return column == getColumnCount() - 1?"Action":layout.getColumns()[column].getName();
    }

    @Override
    public int getRowCount() {
        return games.size();
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Game game = games.get(rowIndex);
        return columnIndex == getColumnCount() -1?game:game.getRow().getColumns()[columnIndex];
    }

    public void addGame(Game game) {
        games.add(game);
        this.fireTableRowsInserted(games.size() - 1, games.size() - 1);
    }

    public void updateGame(Game game) {
        int index = games.indexOf(game);
        this.fireTableRowsUpdated(index, index);
    }

    public void removeGame(Game game) {
        int index = games.indexOf(game);
        games.remove(index);
        this.fireTableRowsDeleted(index, index);
    }

    public void clear() {
        int size = games.size();
        games.clear();
        if (size > 0) {
            this.fireTableRowsDeleted(0, size - 1);
        }
    }
}
