/*
(C) 2007,2008 Michael Chaten, yura.net
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.gametype;

import net.sf.goodgame.client.game.*;
import java.net.URL;
import java.net.URLClassLoader;
import javax.swing.tree.DefaultMutableTreeNode;
import net.sf.goodgame.shared.Util;
import net.sf.goodgame.shared.games.table.GameTableLayout;

/**
 *
 * @author Mike
 */
public class GameType extends DefaultMutableTreeNode {

    public final short ID;
    private final String[] jarNames;
    private final String className;
    private URLClassLoader urlClassLoader;
    private final String name;
    private final GameTableLayout layout;

    public GameType(String name, String jn, String cn, short id, GameTableLayout layout) {

        super(name, true);
        this.name = name;
        ID = id;
        jarNames = Util.split(jn, ',');
        className = cn;
        this.layout = layout;
    }

    public GameTableLayout getLayout() {
        return layout;
    }

    public String[] getJarNames() {

        return jarNames;

    }

    public String getClassName() {

        return className;

    }

    @Override
    public String toString() {
        return name;
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof GameType) {
            return ((GameType) o).ID == ID;
        } else if (o instanceof Short) {
            return ((Short) o).equals(ID);
        }
        return false;
    }

    //Automatically Generated by Netbeans, if there is some reason to change this, please do so.
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 83 * hash + this.ID;
        return hash;
    }

    public GameClient getGameClient(String codeBase) {

        ClassLoader ucl = getClassLoader(codeBase);

        try {

            return (GameClient) Class.forName(className, true, ucl).newInstance();

        } catch (Exception ex) {
            throw new RuntimeException(ex);

        }
    }

    public ClassLoader getClassLoader(String codeBase) {
        if (urlClassLoader == null) { //Lazy init
            try {
                URL[] urls = new URL[jarNames.length];
                for (int i = 0; i < jarNames.length; i++) {
                    urls[i] = new URL("jar:" + codeBase + "/" + jarNames[i] + "!/");
                }
                urlClassLoader = URLClassLoader.newInstance(urls, GameType.class.getClassLoader());
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
        return urlClassLoader;
    }
}
