/*
(C) 2008 Michael Chaten
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.gametype;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import javax.swing.JComponent;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import net.sf.goodgame.client.gui.translucency.TranslucentJList;
import net.sf.goodgame.client.networking.OutgoingConnection;

/**
 *
 * @author mchaten
 */
public class GameTypeListGUI extends TranslucentJList implements GameTypeList,ListSelectionListener {

    private Map<Short,GameType> gameTypeList;
    private OutgoingConnection network;
    public GameTypeListGUI(OutgoingConnection conn)
    {
        super(0.0f);
        network = conn;
        gameTypeList = new ConcurrentHashMap<Short,GameType>();
        this.setCellRenderer(new GameTypeListCellRenderer());
        this.addListSelectionListener(this);
        this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    }
    public void addGameType(GameType gt) {
        gameTypeList.put(gt.ID,gt);
        this.setListData(gameTypeList.values().toArray(new GameType[0]));
    }
    public void reset()
    {
        gameTypeList.clear();
    }
    public GameType getGameType(short id)
    {
        GameType gt = gameTypeList.get(id);
        assert gt.ID == id;
        return gt;
    }

    @Override
    public JComponent getComponent() {
        return this;
    }

    //TODO: Don't change if the previous selected is equal to the current selected.
    //This is reproduced by holding it up to a different game type and then draging it back down to the current one.
    //Not a big bug, but does create unnecessary bandwidth and processing.
    public void valueChanged(ListSelectionEvent event) {
        if(event.getValueIsAdjusting())
            return; //This is called twice per selection change (probably for mouse down->mouse up)

        GameType gt = (GameType)this.getSelectedValue();
        network.switchGameType(gt);
    }
    
    
}
