/*
(C) 2008 Michael Chaten
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.concurrent.ConcurrentHashMap;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.ListSelectionModel;
import net.sf.goodgame.client.gui.translucency.TranslucentJList;
import net.sf.goodgame.client.gui.translucency.TranslucentJPanel;
import net.sf.goodgame.client.networking.OutgoingConnection;

/**
 *
 * @author mchaten
 */
public class LobbyList extends TranslucentJPanel implements MouseListener, ActionListener, LobbyListener{
    
    private DefaultListModel listModel;
    private JPopupMenu popup;
    private JMenuItem switchLobby;
    private OutgoingConnection connection;
    private TranslucentJList list;
    private ConcurrentHashMap<Short,Lobby> lobbyMap;
    public LobbyList(OutgoingConnection connection)
    {
        super(0.5f);
        this.setLayout(new BorderLayout());
        this.connection = connection;
        lobbyMap = new ConcurrentHashMap<Short,Lobby>();
        list = new TranslucentJList(0.5f); 
        list.setLayoutOrientation(JList.HORIZONTAL_WRAP);
        list.setVisibleRowCount(1);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        popup = new JPopupMenu();
        switchLobby = new JMenuItem();
        popup.add(switchLobby);
        popup.setOpaque(false);
        listModel = new DefaultListModel();
        switchLobby.addActionListener(this);
        switchLobby.setOpaque(false);
        list.addMouseListener(this);
        list.setCellRenderer(new LobbyListCellRenderer());
        JLabel lobbyLabel = new JLabel("  Lobby List");
        lobbyLabel.setForeground(Color.white);
        this.add(lobbyLabel,BorderLayout.NORTH);
        this.add(list,BorderLayout.SOUTH);
    }
    public void addLobby(Lobby lobby)
    {
        lobbyMap.put(lobby.ID, lobby);
        listModel.addElement(lobby);
        list.setModel(listModel);
    }
    public Lobby getLobby(short id)
    {
        return lobbyMap.get(id);
    }
    
    public void mouseClicked(MouseEvent e) {
        if(e.getClickCount() >= 2 && e.getButton() == MouseEvent.BUTTON1)
        {
            actionPerformed(null);
        }
    }

    public void mouseEntered(MouseEvent e){
    }

    public void actionPerformed(ActionEvent e) {
        Lobby lobby = (Lobby)list.getSelectedValue();
        connection.switchLobby(lobby.ID);
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        if(e.isPopupTrigger())
        {
            triggerSwitchLobby(e);
        }
    }

    public void mouseReleased(MouseEvent e) {
        if(e.isPopupTrigger())
        {
            triggerSwitchLobby(e);
        }
    }
    private void triggerSwitchLobby(MouseEvent e)
    {
            list.setSelectedIndex(list.locationToIndex(e.getPoint()));
            switchLobby.setText("Switch to the "+list.getSelectedValue()+" Lobby");
            popup.show(e.getComponent(),e.getX(),e.getY());
    }
    public void clearList()
    {
        listModel.clear();
        list.setModel(listModel);
    }
}
