/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package net.sf.goodgame.client.gui;

import java.awt.CardLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import net.sf.goodgame.client.gui.translucency.TranslucentJButton;
import net.sf.goodgame.client.networking.InputOutput;

/**
 *
 * @author mchaten
 */
public class LoginPanel {

    private JButton login;
    private JButton register;
    private InputOutput io;
    private GoodGameUI ui;
    private JPanel root;
    private final CardLayout layout;
    private static final String LOGIN_LOOK = "L";
    private static final String CHOICE_LOOK = "C";
    private static final String REGISTER_LOOK = "R";

    public LoginPanel(GoodGameUI ui, InputOutput io) {
        this.io = io;
        this.ui = ui;
        login = new TranslucentJButton("Login", 0.8f);

        login.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                layout.show(root, LOGIN_LOOK);
            }
        });

        register = new TranslucentJButton("Register", 0.8f);
        register.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                layout.show(root, REGISTER_LOOK);
            }
        });

        JPanel choicePanel = new JPanel();
        choicePanel.setOpaque(false);
        choicePanel.add(login);
        choicePanel.add(register);

        JPanel loginPanel = getLoginPanel();

        JPanel registerPanel = getRegisterPanel();


        root = new JPanel();
        root.setVisible(false);
        root.setLayout(layout = new CardLayout());

        root.add(loginPanel, LOGIN_LOOK);
        root.add(choicePanel, CHOICE_LOOK);
        root.add(registerPanel, REGISTER_LOOK);

        layout.show(root, CHOICE_LOOK);

    }

    private JPanel getLoginPanel() {
        final JLabel usernameLabel = new JLabel("Username");
        final JLabel passwordLabel = new JLabel("Password");
        final JTextField usernameField = new JTextField();
        final JPasswordField passwordField = new JPasswordField();
        JButton ok = new JButton("OK");
        ok.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                login(usernameField.getText(), new String(passwordField.getPassword()));
                usernameField.setText("");
                passwordField.setText("");
            }
        });
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                layout.show(root, CHOICE_LOOK);
            }
        });
        JPanel loginPanel = new JPanel();
        loginPanel.setLayout(new GridLayout(3, 2, 5, 5));
        loginPanel.add(usernameLabel);
        loginPanel.add(usernameField);
        loginPanel.add(passwordLabel);
        loginPanel.add(passwordField);
        loginPanel.add(ok);
        loginPanel.add(cancel);
        return loginPanel;

    }

    private JPanel getRegisterPanel() {
        final JLabel usernameLabel = new JLabel("Username");
        final JLabel password1Label = new JLabel("Password");
        final JLabel password2Label = new JLabel("Confirm Password");
        final JLabel emailLabel = new JLabel("Email Address");
        final JTextField usernameField = new JTextField();
        final JPasswordField password1Field = new JPasswordField();
        final JPasswordField password2Field = new JPasswordField();
        final JTextField emailField = new JTextField();


        JButton ok = new JButton("OK");
        ok.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                register(usernameField.getText(), new String(password1Field.getPassword()), new String(password2Field.getPassword()), emailField.getText());
                usernameField.setText("");
                password1Field.setText("");
                password2Field.setText("");
                emailField.setText("");
            }
        });
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                layout.show(root, CHOICE_LOOK);
            }
        });
        JPanel loginPanel = new JPanel();
        loginPanel.setLayout(new GridLayout(4, 2, 5, 5));
        loginPanel.add(usernameLabel);
        loginPanel.add(usernameField);
        loginPanel.add(password1Label);
        loginPanel.add(password1Field);
        loginPanel.add(password2Label);
        loginPanel.add(password2Field);
        loginPanel.add(emailLabel);
        loginPanel.add(emailField);

        loginPanel.add(ok);
        loginPanel.add(cancel);
        return loginPanel;
    }

    public Component getComponent() {
        return root;
    }

    private void login(String username, String password) {
        username.trim();
        password.trim();
        if(username.isEmpty() || password.isEmpty()) {
            ui.showError("Please fill in all fields");
        } else {
            io.getOutgoingConnection().login(username, password);
        }
    }

    private void register(String username, String password1, String password2, String email) {
        password1.trim();
        username.trim();
        password2.trim();
        email.trim();
        if(username.isEmpty() || password1.isEmpty() || password2.isEmpty() || email.isEmpty()) {
            ui.showError("Please fill in all fields");
        }
        else if (!password1.equals(password2)) {
            ui.showError("Passwords do not match");
        } else {
            io.getOutgoingConnection().register(username, password1, email);
        }
    }
}
