/*
(C) 2007,2008 Michael Chaten, yura.net
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.gui.chat;

import net.sf.goodgame.client.gui.translucency.*;

import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.Color;
import net.sf.goodgame.client.networking.OutgoingConnection;

public class ChatBox extends TranslucentJPanel implements ActionListener, ChatListener {

    
    private String name;
    private JTextArea chat;
    private JTextField chatInput;
    private JLabel header;
    private boolean privatechat;
    private OutgoingConnection network;

    public ChatBox(String name,OutgoingConnection out, boolean priv) {
        
        super(0.5f);  
        this.updateUI();
        network = out;
        privatechat = priv;

        setLayout(new BorderLayout());

        chat = new TranslucentJTextArea(0.4f);
        chat.setEditable(false);

        
        chat.setWrapStyleWord(true);
        chat.setLineWrap(true);     
        chatInput = new TranslucentJTextField(0.0f);
        chatInput.setActionCommand("send");
        chatInput.addActionListener(this);
        chatInput.setForeground(Color.white);
        chatInput.setBackground(new Color(63, 54, 49));
        
        JPanel inchat = new TranslucentJPanel(new BorderLayout(), 0.1f);

        
        JButton chatSend = new TranslucentJButton("Send",0.8f);
        chatSend.setActionCommand("send");
        chatSend.addActionListener(this);

        inchat.add(chatInput);
        inchat.add(chatSend, BorderLayout.EAST);


        header = new JLabel();
        header.setForeground(Color.white);
        setName(name);
        add(header, BorderLayout.NORTH);
        add(new TranslucentJScrollPane(chat, 0.0f));
        add(inchat, BorderLayout.SOUTH);


    }

    @Override
    public void setName(String name) {
        super.setName(name);
        this.name = name;
        if (privatechat) {
            header.setText(" Chat with: " + name);
            
        } else {
            header.setText(" You are currently in the " + name + " Lobby");
        }
    }

    public void actionPerformed(ActionEvent ae) {

        if (ae.getActionCommand().equals("send")) {

            String text = chatInput.getText();

            if (privatechat) {
                incomingChat("me", text);
                network.sendPrivateMessage(name, text);
            } else {
                network.sendChat(text);
            }

            chatInput.setText("");

        } else {

            System.out.println(ae.getActionCommand());
        }

    }

    public void incomingChat(String player, String message) {

        if (player == null) {
            chat.append("*" + message + "*\n");
        } else {
            chat.append(player + ": " + message + "\n");
        }
        chat.setCaretPosition(chat.getDocument().getLength());
    }

}
