/*
    (C) 2007,2008 yura.net
 	This file is part of Lobby.

    Lobby is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License

    Lobby is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.goodgame.client.networking;

import java.util.ResourceBundle;
import java.util.Locale;
import java.util.PropertyResourceBundle;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;

//http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6279809
//This class is needed because ANY call to findResource OR findResources in the URLClassLoader
//cause the jar file to close up, and only reopen, after 50 or so more calls to findResource with a Correct name
//this was the root of the problem "java.lang.IllegalStateException: zip file closed" on the jar file
//when the jar file is closed, classes can not be loaded from it, and so everything crashes!

public class ResBundle extends PropertyResourceBundle {

	private Locale locale;

	public ResBundle(InputStream stream,Locale l) throws IOException {
		super( stream );

		locale = l;
	}

	// may one day want this
        @Override
	public void setParent(ResourceBundle parent) {
		super.setParent(parent);
	}

        @Override
	public Locale getLocale() {
		return locale;
	}


	public static ResBundle getBundle(Class<?> theclass,String name,Locale locale) {

		String loadedlocal = locale.toString();

		int p = loadedlocal.lastIndexOf('_');
		String parent;
		if (p!=-1) { parent=loadedlocal.substring(0,p); }
		else { parent=""; }

		ResBundle resbun;

		try {

			String path = theclass.getPackage().getName().replace('.','/')+"/"+name+(("".equals(loadedlocal))?(""):("_"+loadedlocal)) +".properties";

			URL url = findResource( (URLClassLoader)theclass.getClassLoader(), path );

			if (url==null) {
				throw new Exception("url null");
			}

			resbun = new ResBundle( url.openStream() , locale );

			if (!"".equals(loadedlocal)) {

				ResourceBundle parentbun = getBundle(theclass,name,getLocale(parent));

				resbun.setParent(parentbun);
			}
		}
		catch(Exception ioe) {

			System.out.println("cant find locale: "+loadedlocal+" "+ioe.toString());

			resbun = getBundle(theclass,name,getLocale(parent));

		}

		return resbun;

	}


	public static Locale getLocale(String localeString) {

		if ("".equals(localeString)) { return new Locale(""); }

		String[] splitname = localeString.split("\\_");
		Locale locale;

  			switch(splitname.length) {

			case 1: locale = new Locale(splitname[0]); break;
			case 2: locale = new Locale(splitname[0],splitname[1]); break;
			case 3: locale = new Locale(splitname[0],splitname[1],splitname[2]); break;
			default: throw new IllegalArgumentException("bad local name: "+localeString);
		}

		return locale;

	}

	public static URL findResource(URLClassLoader cl,String name) {

		try {

			URL[] urls = cl.getURLs();

			for (int c=0;c<urls.length;c++) {

				java.net.JarURLConnection conn = (java.net.JarURLConnection)urls[c].openConnection();
				java.util.jar.JarFile jarfile = conn.getJarFile();
				java.util.jar.JarEntry entry = jarfile.getJarEntry(name);

				if (entry!=null) {

					return new URL(urls[c],name);

				}

			}


		}
		catch (Exception ex) {

			ex.printStackTrace();

		}

		return null;
	}

}
