/*
(C) 2008 Michael Chaten
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.user;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Graphics2D;
import javax.swing.DefaultListCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JList;

public class UserListCellRenderer extends DefaultListCellRenderer {

    private static final int gap = 15;
    private boolean drawArraw;
    private String currentPlayer;
    public static final ImageIcon ICON_ARROW = new ImageIcon(UserList.class.getResource("icons/arrow.gif"));
    private boolean isSelected;
    public UserListCellRenderer() {
        currentPlayer = "";
        setOpaque(false);
    }

    @Override
    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {

        Component retValue = super.getListCellRendererComponent(
                list, value, index, isSelected, cellHasFocus);

        User user = (User) value;

        setIcon(user.getIcon());
        //if(retValue instanceof JComponent)
        //	((JComponent)retValue).setOpaque(false);

        drawArraw = (user.toString().equals(currentPlayer));
        this.isSelected = isSelected;
        return retValue;
    }

    @Override
    public void paintComponent(java.awt.Graphics g) {

        if(isSelected)
        {
            Color oldColor = g.getColor();
		Graphics2D g2d = (Graphics2D)g;
		Composite oldComp = g2d.getComposite();
		g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,0.5f));
		g2d.setColor(Color.GREEN);
		g2d.fillRect(0,0,getWidth(),getHeight());
		g2d.setColor(oldColor);
		g2d.setComposite(oldComp);
        }
        if (currentPlayer == null) {

            super.paintComponent(g);
        } else {

            g.translate(gap, 0);

            super.paintComponent(g);

            g.translate(-gap, 0);
        }

        if (drawArraw) {
            g.drawImage(ICON_ARROW.getImage(), 4, (getHeight() - ICON_ARROW.getIconHeight()) / 2, this);
        }
        g.drawRect(TOP, TOP, TOP, TOP);
    }

    // adds a extra bit to the size
    @Override
    public Dimension getPreferredSize() {

        Dimension size = super.getPreferredSize();
        size.setSize(((currentPlayer == null) ? (0) : (gap)) + size.width, size.height);
        return size;
    }
    
    public void setUsername(String username)
    {
        currentPlayer = username;
    }
 }
