/*
(C) 2007,2008 Michael Chaten, yura.net
This file is part of Good Game.
Good Game is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License
Good Game is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.client.user;

import net.sf.goodgame.client.user.userinfo.UserInfoGUI;
import net.sf.goodgame.client.user.userinfo.UserInfoFrame;
import java.awt.event.WindowEvent;
import java.util.List;
import net.sf.goodgame.client.gui.chat.PrivateChatWindow;
import net.sf.goodgame.client.gui.translucency.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.WindowListener;
import javax.swing.DefaultListModel;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import javax.swing.JComponent;
import javax.swing.JLabel;
import net.sf.goodgame.client.gui.chat.PrivateChatController;
import net.sf.goodgame.client.networking.OutgoingConnection;
import net.sf.goodgame.shared.UserInfoElement;
import net.sf.goodgame.shared.UserType;

public class UserListGUI extends TranslucentJPanel implements ActionListener, MouseListener, WindowListener, UserList {

    private JPopupMenu menu;
    private TranslucentJList users;
    private DefaultListModel usersModel;
    private PrivateChatController privateChatController;
    private OutgoingConnection connection;
    private UserListCellRenderer renderer;
    private JMenuItem pchat;
    private JMenuItem info;
    private JMenuItem friend;
    private LinkedList<UserListListener> listeners;
    private Map<String, UserInfoGUI> userInfos;

    public UserListGUI(PrivateChatController controller, OutgoingConnection connection) {
        super(0.5f);
        privateChatController = controller;
        this.connection = connection;
        userInfos = new HashMap<String, UserInfoGUI>();
        setLayout(new BorderLayout());
        listeners = new LinkedList<UserListListener>();
        menu = new JPopupMenu();
        menu.setOpaque(false);
        pchat = new JMenuItem();
        pchat.setActionCommand("pchat");
        pchat.addActionListener(this);
        pchat.setOpaque(false);

        info = new JMenuItem();
        info.setActionCommand("showinfo");
        info.addActionListener(this);
        info.setOpaque(false);

        friend = new JMenuItem();
        friend.setActionCommand("friend");
        friend.addActionListener(this);
        friend.setOpaque(false);

        menu.add(pchat);
        menu.add(info);
        menu.add(friend);

        usersModel = new DefaultListModel();
        users = new TranslucentJList(usersModel, 0.4f);

        users.addMouseListener(this);
        renderer = new UserListCellRenderer();
        users.setCellRenderer(renderer);

        JLabel usersLabel = new JLabel(" Users:");
        usersLabel.setForeground(Color.white);
        add(usersLabel, BorderLayout.NORTH);
        add(new TranslucentJScrollPane(users, 0.0f));

    }

    public void actionPerformed(ActionEvent ae) {

        User user = (User) users.getSelectedValue();

        if (ae.getActionCommand().equals("pchat")) {

            PrivateChatWindow window = privateChatController.getPrivateChatWindow(user.toString());
            window.requestFocus();

        } else if (ae.getActionCommand().equals("showinfo")) {
            connection.getUserInfo(user.getUsername());
        } else if (ae.getActionCommand().equals("friend")) {
            connection.addFriend(user.getUsername());
        } else {
            System.out.print("Unkown Action Command recieved in UserListGUI: ");
            System.out.println(ae.getActionCommand());
        }
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
        if (e.isPopupTrigger()) {
            popupTrigger(e);
        }
    }

    public void mousePressed(MouseEvent e) {
        if (e.isPopupTrigger()) {
            popupTrigger(e);
        }
    }

    private void popupTrigger(MouseEvent e) {
        users.setSelectedIndex(users.locationToIndex(e.getPoint()));
        User user = (User) users.getSelectedValue();
        pchat.setText("Chat with " + user.getUsername());
        info.setText("Get " + user.getUsername() + "'s user information");
        friend.setText("Add " + user.getUsername() + " as a friend");
        menu.show(e.getComponent(), e.getX(), e.getY());
        return;
    }

    public void addUser(User player) {
        usersModel.addElement(player);
        users.setModel(usersModel);
        users.repaint();
        for (UserListListener l : listeners) {
            l.addUser(player);
        }
    }

    public void renameUser(String oldname, String newName, UserType newtype) {
        Enumeration<?> enumer = usersModel.elements();
        for (int index = 0; enumer.hasMoreElements(); index++) {
            if (enumer.nextElement().toString().equals(oldname)) {
                ((User) usersModel.get(index)).rename(newName, newtype);
                users.setModel(usersModel);
                users.revalidate();
                users.repaint();
                for (UserListListener l : listeners) {
                    l.renameUser(oldname, newName);
                }
                return;
            }
        }

        users.repaint();
    }

    public void removeUser(String player) {
        Enumeration<?> enumer = usersModel.elements();
        for (int index = 0; enumer.hasMoreElements(); index++) {
            if (enumer.nextElement().toString().equals(player)) {
                usersModel.removeElementAt(index);
                users.setModel(usersModel);

                for (UserListListener l : listeners) {
                    l.removeUser(player);
                }
                return;
            }
        }
        try {
            throw new Exception("Player not in the list");
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
        users.repaint();
    }

    public void reset() {

        usersModel.clear();
        users.setModel(usersModel);
        users.repaint();
    }

    public void setUsername(String username) {

        renderer.setUsername(username);
        users.repaint();
    }

    public void setUserInfo(String user, List<UserInfoElement> info) {
        UserInfoGUI gui = userInfos.get(user);
        if (gui == null) {
            gui = new UserInfoGUI(user, info);
            userInfos.put(user, gui);
            new UserInfoFrame(gui, this);
        } else {
            gui.setElements(info);
        }
    }

    @Override
    public JComponent getComponent() {
        return this;
    }

    public void addListener(UserListListener listener) {
        listeners.add(listener);
    }

    public void windowClosed(WindowEvent e) {
        UserInfoFrame frame = (UserInfoFrame) e.getWindow();
        userInfos.remove(frame.getUserInfoGUI().getUsername());
    }

    public void windowClosing(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowOpened(WindowEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }
}
