package net.sf.goodgame.client.user.friends;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Graphics2D;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JList;

public class FriendsListCellRenderer extends DefaultListCellRenderer {

    private static final int gap = 15;
    private boolean drawArraw;
    private Friend currentFriend;
    private boolean isSelected;

    public FriendsListCellRenderer() {
        setOpaque(false);
        currentFriend = null;
    }

    @Override
    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {

        Component retValue = super.getListCellRendererComponent(
                list, value, index, isSelected, cellHasFocus);

        currentFriend = (Friend) value;
        this.isSelected = isSelected;
        return retValue;
    }

    @Override
    public void paintComponent(java.awt.Graphics g) {
        Color oldColor;
        if (isSelected) {
            oldColor = g.getColor();
            Graphics2D g2d = (Graphics2D) g;
            Composite oldComp = g2d.getComposite();
            g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
            g2d.setColor(Color.GREEN);
            g2d.fillRect(0, 0, getWidth(), getHeight());
            g2d.setColor(oldColor);
            g2d.setComposite(oldComp);
        }
        if (currentFriend != null) {
            if (!currentFriend.isLoggedIn()) {
                Graphics2D g2d = (Graphics2D) g;
                oldColor = g.getColor();
                Composite oldComp = g2d.getComposite();
                g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
                g2d.setColor(Color.GRAY);
                g2d.fillRect(0, 0, getWidth(), getHeight());
                g2d.setColor(oldColor);
                g2d.setComposite(oldComp);
            }
        }
        super.paintComponent(g);
    }

    // adds a extra bit to the size
    @Override
    public Dimension getPreferredSize() {

        Dimension size = super.getPreferredSize();
        size.setSize(((currentFriend == null) ? (0) : (gap)) + size.width, size.height);
        return size;
    }
}
