package net.sf.goodgame.client.user.friends;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.WindowConstants;
import net.sf.goodgame.client.gui.chat.PrivateChatController;
import net.sf.goodgame.client.gui.chat.PrivateChatWindow;
import net.sf.goodgame.client.networking.OutgoingConnection;

public class FriendsListGUI implements ActionListener, MouseListener {

    private JList guiList;
    private JFrame frame;
    private JPopupMenu menu;
    private JMenuItem pchat;
    private JMenuItem friend;
    private JMenuItem info;
    private PrivateChatController privateChatController;
    private OutgoingConnection connection;

    public FriendsListGUI(PrivateChatController controller, OutgoingConnection connection) {
        privateChatController = controller;
        this.connection = connection;
        guiList = new JList();
        guiList.setCellRenderer(new FriendsListCellRenderer());
        guiList.setPreferredSize(new Dimension(200, 400));
        guiList.addMouseListener(this);
        //root = new JPanel();
        //root.setLayout(new BoxLayout(root, BoxLayout.PAGE_AXIS));
        

        menu = new JPopupMenu();
        menu.setOpaque(false);

        pchat = new JMenuItem();
        pchat.setActionCommand("pchat");
        pchat.addActionListener(this);
        pchat.setOpaque(false);

        info = new JMenuItem();
        info.setActionCommand("showinfo");
        info.addActionListener(this);
        info.setOpaque(false);

        friend = new JMenuItem();
        friend.setActionCommand("friend");
        friend.addActionListener(this);
        friend.setOpaque(false);

        menu.add(pchat);
        menu.add(info);
        menu.add(friend);

        frame = new JFrame("Your Friends List");
        frame.setSize(200, 400);
        frame.setVisible(false);
        frame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
        frame.setContentPane(guiList);
    }

    public void show() {
        frame.setVisible(true);
    }

    public void setFriendsList(FriendsList list) {
        guiList.setModel(list);
    }

    public void actionPerformed(ActionEvent e) {
        Friend selectedFriend = (Friend) guiList.getSelectedValue();
        if (e.getActionCommand().equals("pchat")) {
            PrivateChatWindow window = privateChatController.getPrivateChatWindow(selectedFriend.getUsername());
            window.requestFocus();
        } else if (e.getActionCommand().equals("showinfo")) {
            connection.getUserInfo(selectedFriend.getUsername());
        } else if (e.getActionCommand().equals("friend")) {
            connection.removeFriend(selectedFriend.getUsername());
        } else {
            System.out.print("Unkown Action Command recieved in UserListGUI: ");
            System.out.println(e.getActionCommand());
        }
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
        if (e.isPopupTrigger()) {
            popupTrigger(e);
        }
    }

    public void mousePressed(MouseEvent e) {
        if (e.isPopupTrigger()) {
            popupTrigger(e);
        }
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    private void popupTrigger(MouseEvent e) {
        guiList.setSelectedIndex(guiList.locationToIndex(e.getPoint()));
        Friend myFriend = (Friend) guiList.getSelectedValue();
        pchat.setText("Chat with " + myFriend.getUsername());
        if (myFriend.isLoggedIn()) {
            pchat.setEnabled(true);
        } else {
            pchat.setEnabled(false);
        }
        info.setText("Get " + myFriend.getUsername() + "'s user information");
        friend.setText("Remove " + myFriend.getUsername() + " as a friend");
        menu.show(e.getComponent(), e.getX(), e.getY());
        return;
    }
}
