/*    
(C) 2007,2008 Michael Chaten, yura.net
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.server;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import net.sf.goodgame.server.persistence.InvalidDatabaseException;
import net.sf.goodgame.shared.InvalidConfigurationException;
import net.sf.goodgame.shared.Logging;
import java.io.IOException;
import java.util.Properties;
import net.sf.goodgame.server.administration.AdminGUI;
import net.sf.goodgame.server.lobby.Lobby;
import net.sf.goodgame.server.networking.IOManager;
import net.sf.goodgame.server.networking.InputOutput;
import net.sf.goodgame.server.persistence.Database;
import net.sf.goodgame.server.persistence.DatabaseManager;
import net.sf.goodgame.server.persistence.JPADatabase;
import net.sf.goodgame.server.persistence.MemoryDatabase;
import net.sf.goodgame.shared.Util;

/**
 *
 * @author Mike
 */
public class Main {

    private final InputOutput networking;
    private final Processor processor;
    private static Properties configuration;
    private final static String PORT = "port";
    private final static String BACKLOG = "backlog";
    private final static String DB_TYPE = "database.type";
    private static String CONF_FILE = "gg.conf";
    private final static String MODE = "mode";
    private final static String BACKLOG_DEFAULT = "100";
    private final static String PORT_DEFAULT = "13375";
    private final static String DB_TYPE_MYSQL = "mysql";
    private final static String DB_TYPE_DERBY = "derby";
    private final static String DB_TYPE_MEMORY = "memory";
    private final static String DB_TYPE_DEFAULT = DB_TYPE_MEMORY;
    private final static String DB_PASSWORD = "database.password";
    private final static String DB_USERNAME = "database.username";
    private final static String DB_PORT = "database.port";
    private final static String DB_ADDRESS = "database.address";
    private final static String MODE_DEFAULT = "debug";
    private final static String PROCESSING_BACKLOG_DEFAULT = "100";
    private final static String PROCESSING_BACKLOG = "processing.backlog";

    private static void configSetup() {
        configuration = new Properties();
        FileInputStream fis = null;
        FileOutputStream fos = null;
        try {
            //Find the properties that are not in the file, and give them the default value
            fis = new FileInputStream(CONF_FILE);
            configuration.load(fis);
            if (configuration.getProperty(PORT) == null) {
                configuration.setProperty(PORT, PORT_DEFAULT);
            }
            if (configuration.getProperty(BACKLOG) == null) {
                configuration.setProperty(BACKLOG, BACKLOG_DEFAULT);
            }
            if (configuration.getProperty(DB_TYPE) == null) {
                configuration.setProperty(DB_TYPE, DB_TYPE_DEFAULT);
            }
            if (configuration.getProperty(MODE) == null) {
                configuration.setProperty(MODE, MODE_DEFAULT);
            }
            if (configuration.getProperty(PROCESSING_BACKLOG) == null) {
                configuration.setProperty(PROCESSING_BACKLOG, PROCESSING_BACKLOG_DEFAULT);
            }
        } catch (IOException e) {
            System.err.println("Could not find " + CONF_FILE + ",using defaults");
            configuration.setProperty(PORT, PORT_DEFAULT);
            configuration.setProperty(BACKLOG, BACKLOG_DEFAULT);
            configuration.setProperty(DB_TYPE, DB_TYPE_DEFAULT);
            configuration.setProperty(MODE, MODE_DEFAULT);
            configuration.setProperty(PROCESSING_BACKLOG, PROCESSING_BACKLOG_DEFAULT);
            try {
                fos = new FileOutputStream(CONF_FILE);
                configuration.store(fos, "Good Game Server Config File");
            } catch (IOException io) {
                System.err.println("Error writing out default configuration file");
                io.printStackTrace();
            }
        }
        if (fis != null) {
            try {
                fis.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if(fos != null) {
            try {
                fos.close();
            }catch(IOException e) {
                e.printStackTrace();
            }
        }
        /*
        if (configuration.getProperty(MODE).equalsIgnoreCase("debug"))
        {
        logger.setLevel(Level.ALL); //Change to Level.OFF to turn off logging
        } else
        {
        logger.setLevel(Level.OFF);
        }
        
        logManager = LogManager.getLogManager();
        logManager.addLogger(logger);
        LogManager.getLoggingMXBean().setLoggerLevel(GLOBAL_LOGGER_NAME, Level.ALL.getName());
         */
        Util.getEncodedPublicKey();//Generate the public/private key set before we need it.

    }

    public Main() throws InvalidConfigurationException, InvalidDatabaseException {
        int port;
        int backlog, processing_backlog;
        Database db;
        Util.generateNewKeys();
        try {
            port = Integer.parseInt(configuration.getProperty(PORT));
        } catch (NumberFormatException e) {
            port = Integer.parseInt(PORT_DEFAULT);
        }
        try {
            backlog = Integer.parseInt(configuration.getProperty(BACKLOG));
        } catch (NumberFormatException e) {
            Logging.log("Invalid backlog number, setting to default");
            backlog = Integer.parseInt(BACKLOG_DEFAULT);
        }
        try {
            processing_backlog = Integer.parseInt(configuration.getProperty(PROCESSING_BACKLOG));
        } catch (NumberFormatException e) {
            Logging.log("Invalid Processing backlog number, setting to default");
            processing_backlog = Integer.parseInt(PROCESSING_BACKLOG_DEFAULT);
        }
        String dbType = configuration.getProperty(DB_TYPE);
        if (dbType.equalsIgnoreCase(DB_TYPE_MYSQL)) {
            throw new InvalidConfigurationException("MySQL Databases are not currently supported");
        /*
        String db_username = configuration.getProperty(DB_USERNAME);
        String db_password = configuration.getProperty(DB_PASSWORD);
        if (db_username != null && db_password != null)
        {
        String db_address = configuration.getProperty(DB_ADDRESS);
        String db_port = configuration.getProperty(DB_PORT);
        if (db_address != null && db_port != null)
        {
        db = new DerbyDatabase(db_address, db_port, db_username, db_password);
        } else if (db_address != null)
        {
        db = new DerbyDatabase(db_address, db_username, db_password);
        } else if (db_port != null)
        {
        db = new DerbyDatabase("localhost", db_port, db_username, db_password);
        } else
        {
        db = new DerbyDatabase(db_username, db_password);
        }
        } else
        {
        throw new InvalidConfigurationException("MySQL Database requires a username and password");
        }*/
        } else if (dbType.equalsIgnoreCase(DB_TYPE_MEMORY)) {
            db = new MemoryDatabase();
        } else if (dbType.equalsIgnoreCase(DB_TYPE_DERBY)) {
            db = new JPADatabase();
        } else {
            throw new InvalidDatabaseException("Invalid Database Options");
        }
        processor = new Processor(processing_backlog);
        DatabaseManager.setDatabase(db);
        networking = new InputOutput(port, backlog, processor);
        IOManager.setIO(networking);
        new Lobby("Main");
        AdminGUI admin = new AdminGUI(this);
    }

    public void start() {
        Thread ioThread = new Thread(networking);
        Thread processing = new Thread(processor);
        processing.start();
        ioThread.start();

    }

    public void stop() {
        processor.stop();
        networking.stop();
        DatabaseManager.getDatabase().close();
    }

    public static void main(String[] args) {
        if (args.length > 0) {
            for (int c = 0; c < args.length; c++) {
                if (c == 0) {
                    CONF_FILE = args[c];
                } else {
                    CONF_FILE = CONF_FILE + " " + args[c];
                }
            }
        }
        configSetup();
        try {
            Main server = new Main();
        } catch (InvalidConfigurationException e) {
            System.err.println("Invalid Configuration, please check the config file");
            System.err.println(e.getMessage());
        } catch (InvalidDatabaseException e) {
            System.err.println("Invalid database choice");
            System.err.println(e.getMessage());
        }
    }
}
