/*    
    (C) 2007,2008 Michael Chaten
 	This file is part of GoodGame.

    GoodGame is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License

    GoodGame is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.goodgame.server;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import net.sf.goodgame.server.user.User;

/**
 *
 * @author Mike
 */
public class Processor implements Runnable
{    
    private ExecutorService executor;
    private BlockingQueue<UserRunnable> processingQueue;
    private volatile boolean executing;
    private Thread myThread;
    public Processor(int backlog)
    {
        processingQueue = new ArrayBlockingQueue<UserRunnable>(backlog);
        executor = Executors.newCachedThreadPool();
        executing = false;
    }
    public void stop()
    {
        System.out.println("Closing the processor");
        executing = false;

        processingQueue.clear();
        executor.shutdownNow();
        myThread.interrupt();
        try
        {
            processingQueue.notifyAll();
        }catch(Exception e) {
        }
        System.out.println("Processor Closed");
    }
    @Override
    public void run()
    {
        myThread = Thread.currentThread();
        executing = true;
        while(executing)
        {
            try
            {
                UserRunnable runnable = processingQueue.take();               
                executor.execute(runnable.runnable);
            }catch(InterruptedException e)
            {
                ///Do nothing, interrupted exception comes from processingQueue.take() as it blocks.
            }
        }
    }
    public void execute(final User user,final Runnable command)
    {
        UserRunnable instruction = new UserRunnable(user,command);
        processingQueue.offer(instruction);
    }
}
