/*    
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.server.game;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import net.sf.goodgame.shared.Util;
import net.sf.goodgame.shared.games.table.GameTableLayout;

public abstract class GameType {

    public final String NAME;
    public final short ID;
    public final static GameTypeList gameTypeList = new GameTypeList();
    private String clientJarNames;
    private String clientClassName;
    private String serverJarNames;
    private String serverClassName;

    public GameType(String name) {
        NAME = name;
        ID = (short) gameTypeList.getSize();
    }

    /**
     *
     * @return A new GameServer.
     */
    public abstract GameServer createNewGameServer();

    /**
     * Note: this will be called many times.
     * @return The Game Table Layout for this game type. 
     */
    public abstract GameTableLayout getGameTableLayout();

    public final String getClientJarNames() {
        return clientJarNames;
    }

    public final String getServerClassName() {
        return serverClassName;
    }

    public final String getClientClassName() {
        return clientClassName;
    }

    private final void setServerJarNames(String serverJarNames) {
        this.serverJarNames = serverJarNames;
    }

    public final String getServerJarNames() {
        return serverJarNames;
    }

    private final void setClientJarNames(String jarNames) {
        clientJarNames = jarNames;
    }

    private final void setServerClassName(String className) {
        serverClassName = className;
    }

    private final void setClientClassName(String className) {
        clientClassName = className;
    }

    public static GameType getGameType(short id) {
        return gameTypeList.getGameType(id);
    }

    public static GameType getGameType(String serverJarNames, String serverClassName, String clientJarName, String clientClassName) throws MalformedURLException, ClassNotFoundException, InstantiationException, IllegalAccessException {
        String codeBase = new File(".").toURI().toURL().toString();
        String[] jarNames = Util.split(serverJarNames, ',');
        URL[] urls = new URL[jarNames.length];
        for (int c = 0; c < jarNames.length; c++) {
            urls[c] = new URL("jar:" + codeBase + "/" + jarNames[c] + "!/");
        }
        ClassLoader jarLoader = URLClassLoader.newInstance(urls, ClassLoader.getSystemClassLoader());
        Class<GameType> testClass = (Class<GameType>) jarLoader.loadClass(serverClassName);
        GameType gt = testClass.newInstance();
        gt.setClientClassName(clientClassName);
        gt.setClientJarNames(clientJarName);
        gt.setServerJarNames(serverJarNames);
        gt.setServerClassName(serverClassName);
        gameTypeList.add(gt);
        return gt;
    }
}
