/*    
    (C) 2008 Michael Chaten
 	This file is part of GoodGame.

    GoodGame is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License

    GoodGame is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.goodgame.server.lobby;

import java.util.Map;
import net.sf.goodgame.server.List;
import net.sf.goodgame.server.game.Game;
import net.sf.goodgame.shared.network.ServerToClientHeader;
import java.util.HashMap;
import net.sf.goodgame.server.NumberPool;
import net.sf.goodgame.server.networking.ServerToClientLogic;
import net.sf.goodgame.shared.games.GameState;
import net.sf.goodgame.shared.network.NetworkMessage;

/**
 *
 * @author Mike
 */
public class GameList extends List<Game>{    
    
    private Map<Short,Game> idMap;
    private NumberPool idPool;
    public GameList()
    {
        super(ServerToClientHeader.addGames);
        idPool = new NumberPool(Short.MAX_VALUE);
        idMap = new HashMap<Short,Game>();
    }
    public synchronized void addGame(Game game)
    {
        NetworkMessage networkMessage = ServerToClientLogic.addGames(game);
        idMap.put(game.ID, game);
        super.addData(game, networkMessage);
        super.updateListeners(networkMessage);
    }
    public synchronized void updateGame(Game game)
    {
        NetworkMessage addGame = ServerToClientLogic.addGames(game);
        NetworkMessage modifyGame = ServerToClientLogic.modifyGames(game);
        super.addData(game, addGame);
        super.updateListeners(modifyGame);
    }
    public synchronized Game getGame(short gameID)
    {
        return idMap.get(gameID);
    }
    public synchronized void removeGame(Game game)
    {        
        NetworkMessage networkMessage = ServerToClientLogic.removeGames(game);        
        idMap.remove(game.ID);
        idPool.returnNumber(game.ID);
        super.removeData(game);
        super.updateListeners(networkMessage);
    }
    public synchronized short nextID()
    {
        return (short)idPool.nextNumber();
    }

}
