/*    
    (C) 2008 Michael Chaten
 	This file is part of GoodGame.

    GoodGame is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License

    GoodGame is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.goodgame.server.lobby;

import java.util.HashMap;
import java.util.Map;
import net.sf.goodgame.server.NumberPool;
import net.sf.goodgame.server.game.Game;
import net.sf.goodgame.server.game.GameType;
import net.sf.goodgame.server.user.User;

/**
 *
 * @author Mike
 */
public class Lobby {

    public final short ID;
    public final String NAME;
    private static final LobbyList lobbyList = new LobbyList();
    private static final NumberPool lobbyIDPool = new NumberPool(Short.MAX_VALUE);
    private final Map<GameType, GameList> gameListMap;
    private final UserList userList;

    public static Lobby getLobby(short id) {
        return lobbyList.getLobby(id);
    }
    public static Lobby[] getAllLobbies() {
        return lobbyList.getAll();
    }

    public static void addLobbyListener(User user) {
        lobbyList.addListener(user);
    }

    public static void removeLobbyListener(User user) {
        lobbyList.removeListener(user);
    }

    public Lobby(String name) {
        NAME = name;
        ID = (short) lobbyIDPool.nextNumber();
        assert ID != -1;
        gameListMap = new HashMap<GameType, GameList>();
        for (GameType type : GameType.gameTypeList.getGameTypes()) {
            gameListMap.put(type, new GameList());
        }
        userList = new UserList();
        lobbyList.addLobby(this);
    }

    public Game makeNewGame(User creator) {
            GameType gameType = creator.getCurrentGameType();
            GameList list = gameListMap.get(gameType);
            Game game = new Game(list, (short) list.nextID(), gameType);
            return game;
    }
    
    public void changeUserGameType(User user, GameType gt) {
        GameType currentGameType = user.getCurrentGameType();
        if (currentGameType != null) {
            gameListMap.get(currentGameType).removeListener(user);
        }
        gameListMap.get(gt).addListener(user);
    }
    
    /*
     * @return The game the user joined
     */
    public Game addUserToGame(User user, short id) {
        
        Game game = gameListMap.get(user.getCurrentGameType()).getGame(ID);
        return addUserToGame(user,game);
    }
    public Game addUserToGame(User user,Game game) {
        if(game == null)
            return null;
        boolean added = game.addUser(user);
        if(added)
        {
            this.removeUser(user);
            return game;
        }
        return null;
    }
    public void removeUserFromGame(User user, short id) {
        Game game = gameListMap.get(user.getCurrentGameType()).getGame(ID);
        removeUserFromGame(user,game);
    }
    public void removeUserFromGame(User user,Game game)
    {
        if(game == null)
            return;
        this.addUser(user);
        game.removeUser(user);
    }

    public void addUser(User user) {
        userList.addUser(user);
        userList.addListener(user);
        if (user.getCurrentGameType() != null) {
            gameListMap.get(user.getCurrentGameType()).addListener(user);
        }
    }

    public void modifyUser(User user, String previousName) {
        userList.modifyUser(user, previousName);
    }

    public void removeUser(User user) {
        userList.removeListener(user);
        userList.removeUser(user);
        if (user.getCurrentGameType() != null) {
            gameListMap.get(user.getCurrentGameType()).removeListener(user);
        }
    }

    public void sendChat(User from, String message) {
        userList.sendChat(from, message);
    }
    @Override
    public String toString()
    {
        return this.NAME;
    }
}
