/*    
    (C) 2008 Michael Chaten
 	This file is part of GoodGame.

    GoodGame is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License

    GoodGame is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.goodgame.server.lobby;

import net.sf.goodgame.server.*;
import net.sf.goodgame.server.networking.ServerToClientLogic;
import net.sf.goodgame.server.user.User;
import net.sf.goodgame.shared.network.NetworkMessage;
import net.sf.goodgame.shared.network.ServerToClientHeader;

/**
 *
 * @author Mike
 */
public class UserList extends List<User>
{

    public UserList()
    {
        super(ServerToClientHeader.addUsers);
    }

    public synchronized void addUser(User user)
    {
        NetworkMessage networkMessage = ServerToClientLogic.addUsers(user);
        super.addData(user, networkMessage);
        super.updateListeners(networkMessage);
    }
    
    public synchronized void modifyUser(User user, String previousName)
    {
        NetworkMessage networkMessage = ServerToClientLogic.addUsers(user);
        super.updateData(user, networkMessage);
        networkMessage = ServerToClientLogic.modifyUsers(user,previousName);
        super.updateListeners(networkMessage);
    }

    public synchronized void removeUser(User user)
    {
        super.removeData(user);
        NetworkMessage networkMessage = ServerToClientLogic.removeUsers(user);
        super.updateListeners(networkMessage);
    }

    public synchronized void sendChat(User from, String message)
    {
        NetworkMessage networkMessage = ServerToClientLogic.chat(from, message);
        super.updateListeners(networkMessage);
    }

    public synchronized void sendServerMessage(String message)
    {
        super.updateListeners(ServerToClientLogic.serverMessage(message));
    }
    public synchronized boolean hasUser(User user)
    {
        return super.contains(user);
    }
}
