/*    
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.server.networking;

import java.util.Collection;
import net.sf.goodgame.server.lobby.Lobby;
import net.sf.goodgame.server.game.Game;
import net.sf.goodgame.server.game.GameType;
import net.sf.goodgame.server.persistence.DatabaseManager;
import net.sf.goodgame.server.user.User;
import net.sf.goodgame.shared.UserInfoElement;
import net.sf.goodgame.shared.UserType;
import net.sf.goodgame.shared.Util;
import net.sf.goodgame.shared.network.NetworkMessage;
import net.sf.goodgame.shared.network.ServerToClientHeader;

public class ServerToClientLogic {

    public static NetworkMessage removeFriend(User friend) {
        return NetworkMessage.createMessage(ServerToClientHeader.removeFriend, friend.getUsername());
    }

    private ServerToClientLogic() {
    }
    private static NetworkMessage handShake = NetworkMessage.createMessage(ServerToClientHeader.handshake, Util.getEncodedPublicKey());

    public static NetworkMessage handShake() {
        return handShake;
    }
    private static NetworkMessage oldVersion = NetworkMessage.createMessage(ServerToClientHeader.oldVersion, (Short) ServerToClientHeader.version);

    public static NetworkMessage oldVersion() {
        return oldVersion;
    }

    public static NetworkMessage loginOk(User user) {
        String username = user.getUsername();
        UserType type = user.getType();
        return NetworkMessage.createMessage(ServerToClientHeader.loginOk, type, username);
    }
    private static NetworkMessage loginFailed = NetworkMessage.createMessage(ServerToClientHeader.loginFailed, new byte[0]);

    public static NetworkMessage loginFailed() {
        return loginFailed;
    }

    public static NetworkMessage logout(User user) {
        String username = user.getUsername();
        NetworkMessage networkMessage = NetworkMessage.createMessage(ServerToClientHeader.logout, username);
        return networkMessage;
    }

    public static NetworkMessage switchLobby(Lobby lobby) {
        NetworkMessage networkMessage = NetworkMessage.createMessage(ServerToClientHeader.switchLobby, lobby.ID);
        return networkMessage;
    }

    public static NetworkMessage switchGameType(GameType gt) {
        NetworkMessage networkMessage = NetworkMessage.createMessage(ServerToClientHeader.switchGameType, gt.ID);
        return networkMessage;
    }

    public static NetworkMessage addLobbies(Lobby lobby) {
        return NetworkMessage.createMessage(ServerToClientHeader.addLobbies, lobby.ID, lobby.NAME);
    }

    public static NetworkMessage modifyLobbies(Lobby lobby) {
        return NetworkMessage.createMessage(ServerToClientHeader.modifyLobbies, lobby.ID, lobby.NAME);
    }

    public static NetworkMessage removeLobbies(Lobby lobby) {
        return NetworkMessage.createMessage(ServerToClientHeader.removeLobbies, lobby.ID);
    }

    public static NetworkMessage gameMessage(String message) {
        return NetworkMessage.createMessage(ServerToClientHeader.gameMessage, message);
    }

    public static NetworkMessage gameBytes(byte[] message) {
        return NetworkMessage.createMessage(ServerToClientHeader.gameBytes, message);
    }

    public static NetworkMessage chat(User from, String message) {
        String username = from.getUsername();
        return NetworkMessage.createMessage(ServerToClientHeader.chat, username, message);
    }

    public static NetworkMessage privateMessage(User from, String message) {

        String name = from.getUsername();
        return NetworkMessage.createMessage(ServerToClientHeader.privateMessage, name, message);
    }

    public static NetworkMessage addGames(Game game) {
        return NetworkMessage.createMessage(ServerToClientHeader.addGames,
                game.ID, game.gameType.ID, game.getState(), game.getGameTableRow());
    }

    public static NetworkMessage modifyGames(Game game) {
        return NetworkMessage.createMessage(ServerToClientHeader.modifyGames,
                game.ID, game.getState(), game.getGameTableRow());
    }

    public static NetworkMessage removeGames(Game game) {
        return NetworkMessage.createMessage(ServerToClientHeader.removeGames, game.ID);
    }

    public static NetworkMessage joinGame(Game game) {
        return NetworkMessage.createMessage(ServerToClientHeader.joinGame, game.gameType.ID);
    }
    private static NetworkMessage leaveGame = NetworkMessage.createMessage(ServerToClientHeader.leaveGame, new byte[0]);

    public static NetworkMessage leaveGame() {
        return leaveGame;
    }
    private static NetworkMessage createGame = NetworkMessage.createMessage(ServerToClientHeader.createGame, new byte[0]);

    public static NetworkMessage createGame() {
        return createGame;
    }
    private static NetworkMessage loadGame = NetworkMessage.createMessage(ServerToClientHeader.loadGame, new byte[0]);

    public static NetworkMessage loadGame() {
        return loadGame;
    }

    public static NetworkMessage addUsers(User user) {
        UserType type = user.getType();
        return NetworkMessage.createMessage(ServerToClientHeader.addUsers, type, user.getUsername());
    }

    public static NetworkMessage modifyUsers(User user, String previousName) {
        UserType type = user.getType();
        return NetworkMessage.createMessage(ServerToClientHeader.modifyUsers, type, user.getUsername(), previousName);
    }

    public static NetworkMessage removeUsers(User user) {
        return NetworkMessage.createMessage(ServerToClientHeader.removeUsers, user.getUsername());
    }

    public static NetworkMessage userInformation(String username) {
        Collection<UserInfoElement> userInfos = DatabaseManager.getDatabase().getUserInfo(username);
        if (userInfos == null) {
            return null;
        }
        UserInfoElement[] userInfoArray = userInfos.toArray(new UserInfoElement[0]);
        return NetworkMessage.createMessage(ServerToClientHeader.userInformation, username, userInfoArray);
    }

    public static NetworkMessage addGameTypes(GameType gt) {
        return NetworkMessage.createMessage(ServerToClientHeader.addGames, gt.ID, gt.getClientClassName(), gt.getClientJarNames(),
                gt.NAME, gt.getGameTableLayout());
    }
    private static NetworkMessage modifyGameTypes = NetworkMessage.createMessage(ServerToClientHeader.modifyGameTypes, new byte[0]);

    public static NetworkMessage modifyGameTypes(GameType gt) {
        return modifyGameTypes;
    }
    private static NetworkMessage removeGameTypes = NetworkMessage.createMessage(ServerToClientHeader.removeGameTypes, new byte[0]);

    public static NetworkMessage removeGameTypes() {
        return removeGameTypes;
    }

    public static NetworkMessage serverMessage(String message) {
        return NetworkMessage.createMessage(ServerToClientHeader.serverMessage, message);
    }

    public static NetworkMessage addFriend(String friend) {
        return NetworkMessage.createMessage(ServerToClientHeader.addFriend, friend);
    }

    public static NetworkMessage removeFriend(String friend) {
        return NetworkMessage.createMessage(ServerToClientHeader.removeFriend, friend);
    }

    public static NetworkMessage friendLoggedIn(User friend) {
        return NetworkMessage.createMessage(ServerToClientHeader.friendLoggedIn, friend.getUsername());
    }

    public static NetworkMessage friendLoggedOut(User friend) {
        return NetworkMessage.createMessage(ServerToClientHeader.friendLoggedOut, friend.getUsername());
    }
}
