/*    
    (C) 2007,2008 Michael Chaten
 	This file is part of GoodGame.

    GoodGame is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License

    GoodGame is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.goodgame.server.persistence;

import java.util.Collection;
import net.sf.goodgame.server.game.GameResult;
import net.sf.goodgame.server.game.GameType;
import net.sf.goodgame.server.user.FriendsList;
import net.sf.goodgame.server.user.Results;
import net.sf.goodgame.server.user.User;
import net.sf.goodgame.shared.UserInfoElement;
import net.sf.goodgame.shared.UserType;

public interface Database
{
    /**
     * Determines whether the username has been registered
     * @param username
     * @param password
     * @return
     */
    boolean userIsRegistered(String username);
    
    /**
     * Registers a user with the specified username, password and max class into the database
     * @param username
     * @param password
     * @param maxClass
     */
    public boolean registerUser(String username,String password,String email,UserType type);
    
    /**
     * Gets the privileges from the database for the specified username
     * @param username
     * @return
     */
    public UserType getType(String username);
    /**
     * Logs a person in with the specified username and password.
     * @param username
     * @param password
     * @exception InvalidLoginException thrown if password or username is incorrect, or some other error occured
     * @return
     */
    public boolean login(String username,String password);
    /**
     * Logs a person out with the specified username
     * @param username
     */
    public void logout(String username);
    
    public Collection<UserInfoElement> getUserInfo(String username);
    
    public void saveUserInfo(String username,UserInfoElement info);
    
    public void logGameMove(GameType gameID, String user, String move);
    
    public void addFriend(User user, String friend);
    
    public void removeFriend(User user, String friend);
    
    public FriendsList getFriendsList(User user);
    
    public Collection<String> getFriendsOfList(String username);

    public void storeResults(GameType gt, GameResult r);

    public Results getResults(GameType gt, User u);

    public void storeGameType(GameType gt);

    public String[] getRegisteredUsers();

    public void close();
}
