/*    
(C) 2007,2008 Michael Chaten
This file is part of GoodGame.
GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License
GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.server.persistence;

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.Vector;
import net.sf.goodgame.server.game.GameResult;
import net.sf.goodgame.shared.Logging;
import net.sf.goodgame.server.game.GameType;
import net.sf.goodgame.server.user.FriendsList;
import net.sf.goodgame.server.user.Results;
import net.sf.goodgame.server.user.User;
import net.sf.goodgame.shared.UserInfoElement;
import net.sf.goodgame.shared.UserType;

public class MemoryDatabase implements Database {

    //private LobbyServer lserver;
    private ConcurrentHashMap<String, String> registeredPeople;
    private Vector<String> loggedInPeople;
    private ConcurrentHashMap<String, Set<String>> friendsLists;
    private ConcurrentHashMap<String, Set<String>> friendsOfLists;
    private Map<GameType, Map<User, UserResults>> gameResults;

    public MemoryDatabase() {

        //lserver = ds;
        friendsLists = new ConcurrentHashMap<String, Set<String>>();
        friendsOfLists = new ConcurrentHashMap<String, Set<String>>();
        loggedInPeople = new Vector<String>();
        gameResults = new ConcurrentHashMap<GameType, Map<User, UserResults>>();
        registeredPeople = new ConcurrentHashMap<String, String>();
        /*
        "board.map board.jpg Risk Board Map\n" +
        "aa.map aa.jpg Axis and Allies Map\n" +
        "luca.map luca.jpg Luca Map\n" +
        "risk.map risk.jpg Risk Map\n" +
        "risk2.map risk2.jpg Risk II Map\n" +
        "risk2210.map risk2210.jpg Risk 2210 Map\n" +
        "roman_empire.map roman_empire.jpg Roman Empire Map\n" +
        "sersom.map sersom.jpg S.E.R.S.O.M.\n" +
        "teg.map teg.jpg TEG Map\n" +
        "tube.map tube.jpg Tube Map\n" +
        "uk.map uk.jpg UK Map\n" +
        "world.map world.jpg World Map";
         */

        /*  makeNewGame(
        "Risk",
        "RiskClient.jar",
        "risk.lobby.client.ClientGameRisk",
        "RiskServer.jar",
        "risk.lobby.server.ServerGameRisk",
        "board.map,aa.map,godstorm.map,luca.map,risk.map,risk2.map,risk2210.map,roman_empire.map,sersom.map,teg.map,tube.map,uk.map,world.map");
        makeNewGame(
        "Rescue! Max",
        "Rescue/RescueClient.jar",
        "rescue.lobby.client.ClientGameRescue",
        "Rescue/RescueServer.jar",
        "rescue.lobby.server.ServerGameRescue",
        "rescuemax.xml,rescue.xml,newmax.xml");
        makeNewGame(
        "Blackjack",
        "BlackjackClient.jar",
        "net.yura.blackjack.client.BlackJackClient",
        "BlackjackServer.jar",
        "net.yura.blackjack.server.BlackJack",
        "");
        makeNewGame(
        "Domination",
        "Domination/DominationClient.jar",
        "risk.lobby.client.ClientGameRisk",
        "Domination/DominationServer.jar",
        "risk.lobby.server.ServerGameRisk",
        "luca.map,ameroki.map,eurasien.map,geoscape.map,lotr.map,risk.map,RiskEurope.map,roman_empire.map,sersom.map,teg.map,tube.map,uk.map,world.map");
         */
        /*  makeNewGame("Dummy Test",
        "Dummy.jar",
        "net.sf.goodgame.tbgf.test.client.DummyGameClient",
        "Dummy.jar",
        "net.sf.goodgame.tbgf.test.server.DummyGameServer");
         */
        makeNewGame("Desdemona",
                "Desdemona.jar,Dummy.jar",
                "net.sf.goodgame.tbgf.games.desdemona.client.DesdemonaClient",
                "Desdemona.jar,Dummy.jar",
                "net.sf.goodgame.tbgf.games.desdemona.server.DesdemonaGameType");


    }

    private void makeNewGame(String name, String clientJarName, String clientClassName, String serverJarName, String serverClassName) {
        try {
            GameType gt = GameType.getGameType(serverJarName, serverClassName, clientJarName, clientClassName);
            gameResults.put(gt, new ConcurrentHashMap<User, UserResults>());
        } catch (Exception ex) {
            ex.printStackTrace();
            Logging.log("Error creating server class for " + name, Level.SEVERE);
        }
    }

    public boolean registerUser(String username, String password, String email, net.sf.goodgame.shared.UserType type) {
        if (registeredPeople.containsKey(username)) {
            return false;
        } else {
            registeredPeople.put(username, password);
            return true;
        }
    }

    public boolean userIsRegistered(String username) {
        return registeredPeople.containsKey(username);
    }

    public boolean login(String username, String password) {
        if (password.equals(registeredPeople.get(username))) {
            for (String user : loggedInPeople) {
                if (user.equalsIgnoreCase(username)) {
                    return false;
                }
            }
        } else {
            return false;
        }
        loggedInPeople.add(username);


        return true;
    }

    public FriendsList getFriendsList(User user) {
        Set<String> friends = friendsLists.get(user.getUsername());
        if (friends == null) {
            friends = new HashSet<String>();
            friendsLists.put(user.getUsername(), friends);
        }

        return new FriendsList(user, friends);
    }

    public Set<String> getFriendsOfList(String username) {
        Set<String> friendsOf = friendsOfLists.get(username);
        if (friendsOf == null) {
            friendsOf = new HashSet<String>();
            friendsOfLists.put(username, friendsOf);
        }
        return friendsOf;
    }

    public void logout(String username) {
        loggedInPeople.remove(username);
    }

    public UserType getType(String username) {
        if (registeredPeople.containsKey(username)) {
            return UserType.regular;
        }
        return UserType.guest;
    }

    public Collection<UserInfoElement> getUserInfo(String username) {
        return null;
    }

    public void saveUserInfo(String username, UserInfoElement info) {
        return;
    }

    public void logGameMove(GameType id, String name, String move) {
        //TODO: Implement ? Do we even want to?
    }

    public void addFriend(User user, String friend) {
        Set<String> friendList = friendsLists.get(user.getUsername());
        Set<String> friendsOf = friendsOfLists.get(friend);
        if (friendList == null) {
            friendList = new HashSet<String>();
            friendsLists.put(user.getUsername(), friendList);
        }
        if (friendsOf == null) {
            friendsOf = new HashSet<String>();
            friendsOfLists.put(friend, friendsOf);
        }
        friendList.add(friend);
        friendsOf.add(user.getUsername());
    }

    public void removeFriend(User user, String friend) {
        Set<String> friendList = friendsLists.get(user);
        Set<String> friendsOf = friendsOfLists.get(friend);
        if (friendList != null) {
            friendList.remove(friend);
        }
        if (friendsOf != null) {
            friendsOf.remove(user.getUsername());
        }
    }

    public void storeResults(GameType gt, GameResult r) {
        if (!r.countsTowardsStats()) {
            return;
        }
        Map<User, UserResults> resultMap = gameResults.get(gt);
        if (r.wasTied()) {
            User[] players = r.getTie();
            for (User user : players) {
                UserResults userResults = resultMap.get(user);
                if (userResults == null) {
                    userResults = new UserResults();
                    resultMap.put(user, userResults);
                }
                userResults.addTie();
            }
        } else {
            User[] winners = r.getWinners();
            for (User user : winners) {
                UserResults userResults = resultMap.get(user);
                if (userResults == null) {
                    userResults = new UserResults();
                    resultMap.put(user, userResults);
                }
                userResults.addWin();
            }
            User[] losers = r.getLosers();
            for (User user : losers) {
                UserResults userResults = resultMap.get(user);
                if (userResults == null) {
                    userResults = new UserResults();
                    resultMap.put(user, userResults);
                }
                userResults.addLoss();
            }
        }
    }

    public Results getResults(GameType gt, User user) {
        Map<User, UserResults> resultMap = gameResults.get(gt);
        UserResults userResults = resultMap.get(user);
        if (userResults == null) {
            userResults = new UserResults();
            resultMap.put(user, userResults);
        }
        return userResults;
    }

    public void close() {
    }

    private static class UserResults implements Results {

        private int wins;
        private int losses;
        private int ties;

        public void setWins(int w) {
            System.out.println("Wins: " + w);
            wins = w;
        }

        public void setLosses(int l) {
            System.out.println("Losses: " + l);
            losses = l;
        }

        public void setTies(int t) {
            System.out.println("Ties: " + t);
            ties = t;
        }

        public int getWins() {

            return wins;
        }

        public int getLosses() {
            return losses;
        }

        public int getTies() {
            return ties;
        }

        public void addTie() {
            setTies(ties + 1);
        }

        public void addWin() {
            setWins(wins + 1);
        }

        public void addLoss() {
            setLosses(losses + 1);
        }

        @Override
        public String toString() {
            return "Wins: " + wins + ", Losses: " + losses + ", Ties: " + ties;
        }
    }

    public void storeGameType(GameType gt) {
        return;
    }

    public String[] getRegisteredUsers() {
        return registeredPeople.keySet().toArray(new String[0]);
    }
}
