/*
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sf.goodgame.server.persistence.jpa;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.EntityNotFoundException;
import net.sf.goodgame.server.persistence.jpa.exceptions.NonexistentEntityException;
import net.sf.goodgame.server.persistence.jpa.exceptions.PreexistingEntityException;

/**
 *
 * @author mchaten
 */
public class GameTypeEntityJpaController {

    public GameTypeEntityJpaController(EntityManagerFactory emf) {
        this.emf = emf;
    }
    private EntityManagerFactory emf = null;

    public EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    public void create(GameTypeEntity gameTypeEntity) throws PreexistingEntityException, Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            em.persist(gameTypeEntity);
            em.getTransaction().commit();
        } catch (Exception ex) {
            if (findGameTypeEntity(gameTypeEntity.getName()) != null) {
                throw new PreexistingEntityException("GameTypeEntity " + gameTypeEntity + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void edit(GameTypeEntity gameTypeEntity) throws NonexistentEntityException, Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            gameTypeEntity = em.merge(gameTypeEntity);
            em.getTransaction().commit();
        } catch (Exception ex) {
            String msg = ex.getLocalizedMessage();
            if (msg == null || msg.length() == 0) {
                String id = gameTypeEntity.getName();
                if (findGameTypeEntity(id) == null) {
                    throw new NonexistentEntityException("The gameTypeEntity with id " + id + " no longer exists.");
                }
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void destroy(String id) throws NonexistentEntityException {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            GameTypeEntity gameTypeEntity;
            try {
                gameTypeEntity = em.getReference(GameTypeEntity.class, id);
                gameTypeEntity.getName();
            } catch (EntityNotFoundException enfe) {
                throw new NonexistentEntityException("The gameTypeEntity with id " + id + " no longer exists.", enfe);
            }
            em.remove(gameTypeEntity);
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public List<GameTypeEntity> findGameTypeEntityEntities() {
        return findGameTypeEntityEntities(true, -1, -1);
    }

    public List<GameTypeEntity> findGameTypeEntityEntities(int maxResults, int firstResult) {
        return findGameTypeEntityEntities(false, maxResults, firstResult);
    }

    private List<GameTypeEntity> findGameTypeEntityEntities(boolean all, int maxResults, int firstResult) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from GameTypeEntity as o");
            if (!all) {
                q.setMaxResults(maxResults);
                q.setFirstResult(firstResult);
            }
            return q.getResultList();
        } finally {
            em.close();
        }
    }

    public GameTypeEntity findGameTypeEntity(String id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(GameTypeEntity.class, id);
        } finally {
            em.close();
        }
    }

    public int getGameTypeEntityCount() {
        EntityManager em = getEntityManager();
        try {
            return ((Long) em.createQuery("select count(o) from GameTypeEntity as o").getSingleResult()).intValue();
        } finally {
            em.close();
        }
    }

}
