/*
(C) 2009 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.server.persistence.jpa;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.EntityNotFoundException;
import net.sf.goodgame.server.persistence.jpa.exceptions.NonexistentEntityException;
import net.sf.goodgame.server.persistence.jpa.exceptions.PreexistingEntityException;

/**
 *
 * @author mchaten
 */
public class UserInfoElementEntityJpaController {

    public UserInfoElementEntityJpaController(EntityManagerFactory emf) {
        this.emf = emf;
    }
    private EntityManagerFactory emf = null;

    public EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    public void create(UserInfoElementEntity userInfoElementEntity) throws PreexistingEntityException, Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            em.persist(userInfoElementEntity);
            em.getTransaction().commit();
        } catch (Exception ex) {
            UserInfoElementEntityID id = new UserInfoElementEntityID(userInfoElementEntity);
            if (findUserInfoElementEntity(id) != null) {
                throw new PreexistingEntityException("UserInfoElementEntity " + userInfoElementEntity + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void edit(UserInfoElementEntity userInfoElementEntity) throws NonexistentEntityException, Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            userInfoElementEntity = em.merge(userInfoElementEntity);
            em.getTransaction().commit();
        } catch (Exception ex) {
            String msg = ex.getLocalizedMessage();
            if (msg == null || msg.length() == 0) {
                UserInfoElementEntityID id = new UserInfoElementEntityID(userInfoElementEntity);
                if (findUserInfoElementEntity(id) == null) {
                    throw new NonexistentEntityException("The userInfoElementEntity with id " + id + " no longer exists.");
                }
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void destroy(UserInfoElementEntityID id) throws NonexistentEntityException {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            UserInfoElementEntity userInfoElementEntity;
            try {
                userInfoElementEntity = em.getReference(UserInfoElementEntity.class, id);
                userInfoElementEntity.getColumnName();
                userInfoElementEntity.getUsername();
            } catch (EntityNotFoundException enfe) {
                throw new NonexistentEntityException("The userInfoElementEntity with id " + id + " no longer exists.", enfe);
            }
            em.remove(userInfoElementEntity);
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public List<UserInfoElementEntity> findUserInfoElementEntityEntities() {
        return findUserInfoElementEntityEntities(true, -1, -1);
    }

    public List<UserInfoElementEntity> findUserInfoElementEntityEntities(int maxResults, int firstResult) {
        return findUserInfoElementEntityEntities(false, maxResults, firstResult);
    }

    private List<UserInfoElementEntity> findUserInfoElementEntityEntities(boolean all, int maxResults, int firstResult) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from UserInfoElementEntity as o");
            if (!all) {
                q.setMaxResults(maxResults);
                q.setFirstResult(firstResult);
            }
            return q.getResultList();
        } finally {
            em.close();
        }
    }

    public List<UserInfoElementEntity> findUserInfo(String username)
    {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from UserInfoElementEntity as o where o.username=?1").setParameter(1, username);
            return q.getResultList();
        } finally {
            em.close();
        }
    }
    public UserInfoElementEntity findUserInfoElementEntity(UserInfoElementEntityID id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(UserInfoElementEntity.class, id);
        } finally {
            em.close();
        }
    }

    public int getUserInfoElementEntityCount() {
        EntityManager em = getEntityManager();
        try {
            return ((Long) em.createQuery("select count(o) from UserInfoElementEntity as o").getSingleResult()).intValue();
        } finally {
            em.close();
        }
    }
}
