/*
(C) 2008 Michael Chaten
This file is part of GoodGame.

GoodGame is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License

GoodGame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.goodgame.server.persistence.jpa;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.EntityNotFoundException;
import net.sf.goodgame.server.persistence.jpa.exceptions.NonexistentEntityException;
import net.sf.goodgame.server.persistence.jpa.exceptions.PreexistingEntityException;

/**
 *
 * @author mchaten
 */
public class UserResultsEntityJpaController {

    public UserResultsEntityJpaController(EntityManagerFactory emf) {
        this.emf = emf;
    }
    private EntityManagerFactory emf = null;

    public EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    public void create(UserResultsEntity userResultsEntity) throws PreexistingEntityException, Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            em.persist(userResultsEntity);
            em.getTransaction().commit();
        } catch (Exception ex) {
            UserResultsEntityID id = new UserResultsEntityID(userResultsEntity);
            if (findUserResultsEntity(id) != null) {
                throw new PreexistingEntityException("UserResultsEntity " + userResultsEntity + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void edit(UserResultsEntity userResultsEntity) throws NonexistentEntityException, Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            userResultsEntity = em.merge(userResultsEntity);
            em.getTransaction().commit();
        } catch (Exception ex) {
            String msg = ex.getLocalizedMessage();
            if (msg == null || msg.length() == 0) {
                UserResultsEntityID id = new UserResultsEntityID(userResultsEntity);
                if (findUserResultsEntity(id) == null) {
                    throw new NonexistentEntityException("The userResultsEntity with id " + id + " no longer exists.");
                }
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void destroy(UserResultsEntityID id) throws NonexistentEntityException {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            UserResultsEntity userResultsEntity;
            try {
                userResultsEntity = em.getReference(UserResultsEntity.class, id);
                userResultsEntity.getUsername();
                userResultsEntity.getGameType();
            } catch (EntityNotFoundException enfe) {
                throw new NonexistentEntityException("The userResultsEntity with id " + id + " no longer exists.", enfe);
            }
            em.remove(userResultsEntity);
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public List<UserResultsEntity> findUserResultsEntityEntities() {
        return findUserResultsEntityEntities(true, -1, -1);
    }

    public List<UserResultsEntity> findUserResultsEntityEntities(int maxResults, int firstResult) {
        return findUserResultsEntityEntities(false, maxResults, firstResult);
    }

    private List<UserResultsEntity> findUserResultsEntityEntities(boolean all, int maxResults, int firstResult) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from UserResultsEntity as o");
            if (!all) {
                q.setMaxResults(maxResults);
                q.setFirstResult(firstResult);
            }
            return q.getResultList();
        } finally {
            em.close();
        }
    }

    public UserResultsEntity findUserResultsEntity(UserResultsEntityID id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(UserResultsEntity.class, id);
        } finally {
            em.close();
        }
    }

    public List<UserResultsEntity> findUserResultsEntity(String username) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from UserResultsEntity as o where o.username=?1").setParameter(1, username);
            return q.getResultList();
        } finally {
            em.close();
        }
    }

    public int getUserResultsEntityCount() {
        EntityManager em = getEntityManager();
        try {
            return ((Long) em.createQuery("select count(o) from UserResultsEntity as o").getSingleResult()).intValue();
        } finally {
            em.close();
        }
    }
}
