package net.sf.goodgame.shared.games;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import net.sf.goodgame.shared.Util;
import net.sf.goodgame.shared.network.InvalidMessageException;

/**
 *
 * @author mchaten
 */
public class GameOptions extends HashMap<String, byte[]> {

    private byte[] networkMessage = null;

    public String getAsString(String fieldName) {
        byte[] array = getAsArray(fieldName);
        if (array != null) {
            return new String(array);
        } else {
            return null;
        }
    }

    public byte[] getAsArray(String fieldName) {
        return super.get(fieldName);
    }

    public int getAsInt(String fieldName) {
        byte[] array = getAsArray(fieldName);
        if (array != null) {
            try {
                return Util.byteArrayToInt(array);
            } catch (InvalidMessageException e) {
                return -1;
            }
        } else {
            return -1;
        }
    }

    public short getAsShort(String fieldName) {
        byte[] array = getAsArray(fieldName);
        if (array != null) {
            try {
                return Util.byteArrayToShort(array);
            } catch (InvalidMessageException ex) {
                return -1;
            }
        } else {
            return -1;
        }

    }

    public byte getAsByte(String fieldName) {
        byte[] array = getAsArray(fieldName);
        if (array != null && array.length > 0) {
            return array[0];
        } else {
            return -1;
        }
    }

    public synchronized byte[] put(String fieldName, int value) {
        networkMessage = null;
        return super.put(fieldName, Util.intToByteArray(value));
    }

    public synchronized byte[] put(String fieldName, short value) {
        networkMessage = null;
        return super.put(fieldName, Util.shortToByteArray(value));
    }

    public synchronized byte[] put(String fieldName, byte value) {
        networkMessage = null;
        return super.put(fieldName, new byte[]{value});
    }

    public synchronized byte[] put(String fieldName, String value) {
        networkMessage = null;
        return super.put(fieldName, value.getBytes());
    }

    @Override
    public synchronized byte[] put(String fieldname, byte[] value) {
        networkMessage = null;
        return super.put(fieldname, value);
    }
    @Override
    public synchronized void putAll(Map<? extends String,? extends byte[]> m)
    {
        networkMessage = null;
        super.putAll(m);
    }
    public synchronized byte[] getNetworkMessage() {
        if (networkMessage == null) {
            networkMessage = new byte[0];
            for (Entry<String, byte[]> entry : this.entrySet()) {
                byte[] entryMessage;
                byte[] key = entry.getKey().getBytes();
                byte[] keyLen = new byte[]{(byte) key.length};
                byte[] value = entry.getValue();
                byte[] valueLen = Util.shortToByteArray((short) value.length);
                entryMessage = Util.combineArrays(keyLen, key, valueLen, value);
                networkMessage = Util.combineArrays(networkMessage, entryMessage);
            }
        }
        return networkMessage;
    }

    public static GameOptions getGameOptions(byte[] networkMessage, short read) throws InvalidMessageException {
        GameOptions options = new GameOptions();
        while (networkMessage.length > read) {
            byte keyLen = networkMessage[read++];
            if (networkMessage.length >= read + keyLen) {
                String key = Util.extractString(networkMessage, read, keyLen);
                read += keyLen;
                if (networkMessage.length >= read + 2) {
                    short valueLen;
                    valueLen = Util.byteArrayToShort(networkMessage, read);
                    read += 2;
                    if (networkMessage.length >= read + valueLen) {
                        byte[] value = Util.extractArray(networkMessage, read, valueLen);
                        read+=valueLen;
                        options.put(key, value);
                    }
                }
            }
        }
        return options;
    }
}
