/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package net.sf.goodgame.shared.network;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import net.sf.goodgame.shared.UserInfoElement;
import net.sf.goodgame.shared.UserInfoType;
import net.sf.goodgame.shared.games.GameState;
import net.sf.goodgame.shared.UserType;
import net.sf.goodgame.shared.Util;
import net.sf.goodgame.shared.games.GameOptions;
import net.sf.goodgame.shared.games.table.GameTableColumn;
import net.sf.goodgame.shared.games.table.GameTableLayout;
import net.sf.goodgame.shared.games.table.GameTableRow;

/**
 *
 * @author Mike
 */
public class ImmutableNetworkMessage extends NetworkMessage {

    private final byte[] header;
    private byte[] payload;
    private ServerToClientHeader stcHeader;
    private ClientToServerHeader ctsHeader;

    public ImmutableNetworkMessage(ServerToClientHeader enumHeader, byte[] header, byte[] payload) {
        this(header, payload);
        stcHeader = enumHeader;
        ctsHeader = null;
    }

    public ImmutableNetworkMessage(ClientToServerHeader enumHeader, byte[] header, byte[] payload) {
        this(header, payload);
        ctsHeader = enumHeader;
        stcHeader = null;
    }

    private ImmutableNetworkMessage(byte[] header, byte[] payload) {
        this.payload = payload;
        this.header = header;
    }

    @Override
    public void decrypt() {
        payload = Util.decrypt(payload);
        if (header != null) {
            short payloadLen = (short) payload.length;
            byte[] payloadLenArray = Util.shortToByteArray(payloadLen);
            header[2] = payloadLenArray[0];
            header[3] = payloadLenArray[1];
        }
    }

    @Override
    public void encrypt() {
        payload = Util.encrypt(payload);
        if (header != null) {
            short payloadLen = (short) payload.length;
            byte[] payloadLenArray = Util.shortToByteArray(payloadLen);
            header[2] = payloadLenArray[0];
            header[3] = payloadLenArray[1];
        }
    }

    @Override
    public byte[] getHeader() {
        return header;
    }

    @Override
    public byte[] getPayload() {
        return payload;
    }

    /**
     * Valid Data Types are:
     * Boolean, Byte, Int, Short, String, GameOptions (Only as last data type)
     * @param message The message as recieved from the tubes minus the header.
     * @param dataTypes The types of data represented in the message inorder.
     * @return The data in the message.
     */
    public Object[] decodeMessage(boolean list, Class... dataTypes) throws InvalidMessageException {
        short read = 0;
        ArrayList<Object> data = new ArrayList<Object>();
        data.ensureCapacity(dataTypes.length);
        System.out.println("Length: " + payload.length);
        /* Start: i = 0,
         * While: read is less than payload.length and either it is a list or i is less than dataTypes.length
         * Each Iteration: i = if list and i equals (dataTypes.length -1): 0 otherwise i+1
         */
        for (int i = 0; read < payload.length && (i < dataTypes.length || list);
                i = (list && i == dataTypes.length - 1 ? 0 : i + 1)) {
            if (dataTypes[i] == String.class) {
                short stringLength = Util.byteArrayToShort(payload, read);
                read += 2;
                if (read + stringLength > payload.length) {
                    throw new InvalidMessageException("String length is too long");
                }
                String str = Util.extractString(payload, read, stringLength);
                data.add(str);
                read += stringLength;
            } else if (dataTypes[i] == GameOptions.class && !list) {
                if (i + 1 != dataTypes.length) {
                    throw new InvalidParameterException("Game Options must be at the end of the message");
                }
                data.add(GameOptions.getGameOptions(payload, read));
            } else if (dataTypes[i] == Byte.class) {
                data.add(payload[read++]);
            } else if (dataTypes[i] == Short.class) {
                data.add(Util.byteArrayToShort(payload, read));
                read += 2;
            } else if (dataTypes[i] == Integer.class) {
                data.add(Util.byteArrayToInt(payload, read));
                read += 4;
            } else if (dataTypes[i] == Boolean.class) {
                data.add(payload[read++] == 0 ? false : true);
            } else if (dataTypes[i] == GameState.class) {
                GameState state = GameState.getState(payload[read++]);
                data.add(state);
            } else if (dataTypes[i] == new byte[0].getClass()) {
                short arrayLen = Util.byteArrayToShort(payload, read);
                read += 2;
                byte[] array = Util.extractArray(payload, read, arrayLen);
                read += arrayLen;
                if (array == null) {
                    throw new InvalidMessageException("Array Length too long");
                }
                data.add(array);
            } else if (dataTypes[i] == UserType.class) {
                UserType type = UserType.getType(payload[read++]);
                data.add(type);
            } else if (dataTypes[i] == GameTableRow.class) {
                short len = Util.byteArrayToShort(payload, read);
                read += 2;
                len += read; //The position to stop reading at is len.
                ArrayList<String> columns = new ArrayList<String>();
                while (read < len) {
                    short columnLen = Util.byteArrayToShort(payload, read);
                    read += 2;
                    String column = Util.extractString(payload, read, columnLen);
                    if (column == null) {
                        throw new InvalidMessageException("Invalid format");
                    }
                    read += columnLen;
                    columns.add(column);
                }
                if (read > len) {
                    throw new InvalidMessageException("Invalid format");
                }
                data.add(new GameTableRow(columns.toArray(new String[0])));
            } else if (dataTypes[i] == GameTableLayout.class) {
                short len = Util.byteArrayToShort(payload, read);

                read += 2;
                len += read;
                ArrayList<GameTableColumn> columns = new ArrayList<GameTableColumn>();
                while (read < len) {
                    byte size = payload[read++];
                    short nameLen = Util.byteArrayToShort(payload, read);
                    read += 2;
                    String name = Util.extractString(payload, read, nameLen);
                    if (name == null) {
                        throw new InvalidMessageException("Invalid format");
                    }
                    read += nameLen;
                    columns.add(new GameTableColumn(name, size));
                }
                if (read > len) {
                    throw new InvalidMessageException("Invalid format");
                }
                data.add(new GameTableLayout(columns.toArray(new GameTableColumn[0])));
            } else if (dataTypes[i] == UserInfoElement.class || dataTypes[i] == UserInfoElement[].class) {
                short numElements;
                if (dataTypes[i] == UserInfoElement.class) {
                    numElements = 1;
                } else {
                    numElements = Util.byteArrayToShort(payload, read);
                    read += 2;
                }
                UserInfoElement[] elements = new UserInfoElement[numElements];
                for(int j = 0;j<numElements;j++)
                {
                    short columnLen = Util.byteArrayToShort(payload, read);
                    read += 2;
                    String columnName = Util.extractString(payload, read, columnLen);
                    read += columnLen;
                    short textLen = Util.byteArrayToShort(payload, read);
                    read += 2;
                    String text = Util.extractString(payload, read, textLen);
                    read += textLen;
                    UserInfoType type = UserInfoType.getType(payload[read++]);
                    if (columnName == null || text == null || type == null) {
                        throw new InvalidMessageException("Invalid format");
                    }
                    elements[j] = new UserInfoElement(columnName, text, type);
                }
                if(dataTypes[i] == UserInfoElement.class) {
                    data.add(elements[0]);
                } else
                {
                    data.add(elements);
                }
                

            } else {
                throw new InvalidParameterException("Unknown data type: " + dataTypes[i]);
            }
        }
        if (data.size() % dataTypes.length == 0) {
            return data.toArray(new Object[0]);
        } else {
            throw new InvalidMessageException("List was not complete, got: " + data.size() +
                    " when we needed a multiple of: " + dataTypes.length);
        }
    }

    public String toString() {
        if (stcHeader == null) {
            return ctsHeader.toString();
        }
        return stcHeader.toString();
    }
}
