/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package net.sf.goodgame.shared.network;

import java.security.InvalidParameterException;
import java.util.HashMap;
import net.sf.goodgame.shared.Util;
import net.sf.goodgame.shared.games.GameOptions;

/**
 * @author Mike
 */
public class MutableNetworkMessage<A> extends NetworkMessage {

    private HashMap<A, byte[]> payload;
    private byte[] header;
    private ServerToClientHeader enumHeader;
    private short payloadLen;

    public MutableNetworkMessage(ServerToClientHeader header) {
        this.header = new byte[]{header.messageNumberArray[0], header.messageNumberArray[1], 0, 0};
        payload = new HashMap<A, byte[]>();
        enumHeader = header;
        payloadLen = 0;
    }

    @Override
    public void decrypt() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void encrypt() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public byte[] getHeader() {
        return header;
    }

    @Override
    public byte[] getPayload() {
        return Util.combineArrays(payload.values().toArray(new byte[0][0]));
    }

    /**
     * Valid Data Types are:
     * Boolean, Byte, Int, Short, String, GameOptions (Only as last data type)
     * @param message The message as recieved from the tubes minus the header.
     * @param dataTypes The types of data represented in the message inorder.
     * @return The data in the message.
     */
    public Object[] decodeMessage(boolean list, Class... dataTypes) throws InvalidMessageException {
        short read = 0;
        Object[] data = new Object[dataTypes.length];
        for (byte[] curPayload : payload.values()) {
            for (int i = 0; read < curPayload.length && i < dataTypes.length; i++) {
                if (dataTypes[i] == String.class) {
                    byte stringLength = curPayload[read];
                    read += 1;
                    if (read + stringLength >= curPayload.length) {
                        throw new InvalidMessageException("String length is too long");
                    }
                    data[i] = Util.extractString(curPayload, read, stringLength);
                    read += stringLength;
                } else if (dataTypes[i] == GameOptions.class) {
                    if (i + 1 != dataTypes.length) {
                        throw new InvalidParameterException("Game Options must be at the end of the message");
                    }
                    data[i] = GameOptions.getGameOptions(curPayload, read);
                } else if (dataTypes[i] == Byte.class) {
                    data[i] = curPayload[read];
                    read += 1;
                } else if (dataTypes[i] == Short.class) {
                    data[i] = Util.byteArrayToShort(curPayload, read);
                    read += 2;
                } else if (dataTypes[i] == Integer.class) {
                    data[i] = Util.byteArrayToInt(curPayload, read);
                    read += 4;
                } else if (dataTypes[i] == Boolean.class) {
                    data[i] = (curPayload[read] == 0 ? false : true);
                    read += 1;
                } else {
                    throw new InvalidParameterException("Unknown data type: " + dataTypes[i]);
                }
            }
        }
        return data;
    }

    public void put(A key, NetworkMessage message) {
        byte[] messagePayload = message.getPayload();
        byte[] oldValue = payload.put(key, messagePayload);
        if (oldValue != null) {
            payloadLen -= oldValue.length;
        }
        payloadLen += (short) messagePayload.length;
        byte[] payloadLenArray = Util.shortToByteArray(payloadLen);
        header[2] = payloadLenArray[0];
        header[3] = payloadLenArray[1];
    }

    public void remove(Object key) {
        byte[] message = payload.remove(key);
        if (message == null) {
            System.out.println("Invalid key: " + key);
            return;
        }
        payloadLen -= (short) message.length;
        byte[] payloadLenArray = Util.shortToByteArray(payloadLen);
        header[2] = payloadLenArray[0];
        header[3] = payloadLenArray[1];
    }

    @Override
    public String toString() {
        return enumHeader.toString();
    }
}
